"""Convenient access to an SQLObject or SQLAlchemy managed database."""

import sys
import time
import logging

import cherrypy
from cherrypy import request
from cherrypy.filters.basefilter import BaseFilter

try:
    import sqlalchemy
    from sqlalchemy.orm import create_session as orm_create_session
except ImportError:
    sqlalchemy = None

try:
    import sqlobject
    from sqlobject.dbconnection import ConnectionHub, Transaction, TheURIOpener
    from sqlobject.util.threadinglocal import local as threading_local
except ImportError:
    sqlobject = None

import dispatch
from turbogears import config
from turbogears.util import remove_keys
from turbogears.genericfunctions import MultiorderGenericFunction

log = logging.getLogger("turbogears.database")

_engine = None

# Provide support for SQLAlchemy
if sqlalchemy:
    def get_engine():
        """Retrieve the engine based on the current configuration."""
        global _engine
        if not _engine:
            alch_args = dict()
            for k, v in config.config.configMap["global"].items():
                if "sqlalchemy" in k:
                    alch_args[k.split(".")[-1]] = v
            dburi = alch_args.pop('dburi')
            if not dburi:
                raise KeyError("No sqlalchemy database config found!")
            _engine = sqlalchemy.create_engine(dburi, **alch_args)
        if not metadata.is_bound():
            metadata.bind = _engine
        return _engine

    def create_session():
        """Create a session that uses the engine from thread-local metadata."""
        if not metadata.is_bound():
            get_engine()
        return orm_create_session()

    metadata = sqlalchemy.MetaData()
    try:
        from sqlalchemy.orm import scoped_session
        # Create session with autoflush=False
        # and autocommit=True (transactional=False)
        session = scoped_session(create_session)
        mapper = session.mapper # use session-aware mapper
    except ImportError: # SQLAlchemy < 0.4
        from sqlalchemy.ext.sessioncontext import SessionContext
        class Objectstore(object):
            def __init__(self):
                self.context = SessionContext(create_session)
            def __getattr__(self, name):
                return getattr(self.context.registry(), name)
            session = property(lambda s: s.context.registry())
        session = Objectstore()
        context = session.context
        Query = sqlalchemy.Query
        from sqlalchemy.orm import mapper as orm_mapper
        def mapper(cls, *args, **kwargs):
            validate = kwargs.pop('validate', False)
            if not hasattr(getattr(cls, '__init__'), 'im_func'):
                def __init__(self, **kwargs):
                     for key, value in kwargs.items():
                         if validate and key not in self.mapper.props:
                             raise KeyError(
                                "Property does not exist: '%s'" % key)
                         setattr(self, key, value)
                cls.__init__ = __init__
            m = orm_mapper(cls, extension=context.mapper_extension,
                *args, **kwargs)
            class query_property(object):
                def __get__(self, instance, cls):
                    return Query(cls, session=context.current)
            cls.query = query_property()
            return m

    try:
        from sqlalchemy.ext import activemapper
        activemapper.metadata, activemapper.objectstore = metadata, session
    except ImportError:
        pass
    try:
        import elixir
        elixir.metadata, elixir.session = metadata, session
    except ImportError:
        pass

else:
    def get_engine():
        pass

    def create_session():
        pass

    metadata = session = mapper = None

bind_meta_data = bind_metadata = get_engine # alias names

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

hub_registry = set()

_hubs = dict() # stores the AutoConnectHubs used for each connection URI

# Provide support for SQLObject
if sqlobject:
    def _mysql_timestamp_converter(raw):
        """Convert a MySQL TIMESTAMP to a floating point number representing
        the seconds since the Un*x Epoch. It uses custom code the input seems
        to be the new (MySQL 4.1+) timestamp format, otherwise code from the
        MySQLdb module is used."""
        if raw[4] == '-':
            return time.mktime(time.strptime(raw, '%Y-%m-%d %H:%M:%S'))
        else:
            import MySQLdb.converters
            return MySQLdb.converters.mysql_timestamp_converter(raw)


    class AutoConnectHub(ConnectionHub):
        """Connects to the database once per thread. The AutoConnectHub also
        provides convenient methods for managing transactions."""
        uri = None
        params = {}

        def __init__(self, uri=None, supports_transactions=True):
            if not uri:
                uri = config.get("sqlobject.dburi")
            self.uri = uri
            self.supports_transactions = supports_transactions
            hub_registry.add(self)
            ConnectionHub.__init__(self)

        def _is_interesting_version(self):
            """Return True only if version of MySQLdb <= 1.0."""
            import MySQLdb
            module_version = MySQLdb.version_info[0:2]
            major = module_version[0]
            minor = module_version[1]
            # we can't use Decimal here because it is only available for Python 2.4
            return (major < 1 or (major == 1 and minor < 2))

        def _enable_timestamp_workaround(self, connection):
            """Enable a workaround for an incompatible timestamp format change
            in MySQL 4.1 when using an old version of MySQLdb. See trac ticket
            #1235 - http://trac.turbogears.org/ticket/1235 for details."""
            # precondition: connection is a MySQLConnection
            import MySQLdb
            import MySQLdb.converters
            if self._is_interesting_version():
                conversions = MySQLdb.converters.conversions.copy()
                conversions[MySQLdb.constants.FIELD_TYPE.TIMESTAMP] = \
                    _mysql_timestamp_converter
                # There is no method to use custom keywords when using
                # "connectionForURI" in sqlobject so we have to insert the
                # conversions afterwards.
                connection.kw["conv"] = conversions

        def getConnection(self):
            try:
                conn = self.threadingLocal.connection
                return self.begin(conn)
            except AttributeError:
                if self.uri:
                    conn = sqlobject.connectionForURI(self.uri)
                    # the following line effectively turns off the DBAPI connection
                    # cache. We're already holding on to a connection per thread,
                    # and the cache causes problems with sqlite.
                    if self.uri.startswith("sqlite"):
                        TheURIOpener.cachedURIs = {}
                    elif self.uri.startswith("mysql") and \
                         config.get("turbogears.enable_mysql41_timestamp_workaround", False):
                        self._enable_timestamp_workaround(conn)
                    self.threadingLocal.connection = conn
                    return self.begin(conn)
                raise AttributeError(
                    "No connection has been defined for this thread "
                    "or process")

        def reset(self):
            """Used for testing purposes. This drops all of the connections
            that are being held."""
            self.threadingLocal = threading_local()

        def begin(self, conn=None):
            """Start a transaction."""
            if not self.supports_transactions:
                return conn
            if not conn:
                conn = self.getConnection()
            if isinstance(conn, Transaction):
                if conn._obsolete:
                    conn.begin()
                return conn
            self.threadingLocal.old_conn = conn
            trans = conn.transaction()
            self.threadingLocal.connection = trans
            return trans

        def commit(self):
            """Commit the current transaction."""
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if isinstance(conn, Transaction):
                self.threadingLocal.connection.commit()

        def rollback(self):
            """Rollback the current transaction."""
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if isinstance(conn, Transaction) and not conn._obsolete:
                self.threadingLocal.connection.rollback()

        def end(self):
            """End the transaction, returning to a standard connection."""
            if not self.supports_transactions:
                return
            try:
                conn = self.threadingLocal.connection
            except AttributeError:
                return
            if not isinstance(conn, Transaction):
                return
            if not conn._obsolete:
                conn.rollback()
            self.threadingLocal.connection = self.threadingLocal.old_conn
            del self.threadingLocal.old_conn
            self.threadingLocal.connection.cache.clear()

    class PackageHub(object):
        """Transparently proxies to an AutoConnectHub for the URI
        that is appropriate for this package. A package URI is
        configured via "packagename.dburi" in the global CherryPy
        settings. If there is no package DB URI configured, the
        default (provided by "sqlobject.dburi") is used.

        The hub is not instantiated until an attempt is made to
        use the database.
        """
        def __init__(self, packagename):
            self.packagename = packagename
            self.hub = None

        def __get__(self, obj, type):
            if self.hub:
                return self.hub.__get__(obj, type)
            else:
                return self

        def __set__(self, obj, type):
            if not self.hub:
                self.set_hub()
            return self.hub.__set__(obj, type)

        def __getattr__(self, name):
            if not self.hub:
                self.set_hub()
            try:
                return getattr(self.hub, name)
            except AttributeError:
                return getattr(self.hub.getConnection(), name)

        def set_hub(self):
            dburi = config.get("%s.dburi" % self.packagename, None)
            if not dburi:
                dburi = config.get("sqlobject.dburi", None)
            if not dburi:
                raise KeyError, "No database configuration found!"
            if dburi.startswith("notrans_"):
                dburi = dburi[8:]
                trans = False
            else:
                trans = True
            hub = _hubs.get(dburi, None)
            if not hub:
                hub = AutoConnectHub(dburi, supports_transactions=trans)
                _hubs[dburi] = hub
            self.hub = hub
else:
    class AutoConnectHub(object):
        pass

    class PackageHub(object):
        pass

def set_db_uri(dburi, package=None):
    """Sets the database URI to use either globally or for a specific
    package. Note that once the database is accessed, calling
    setDBUri will have no effect.

    @param dburi: database URI to use
    @param package: package name this applies to, or None to set the default.
    """
    if package:
        config.update({'global':
            {"%s.dburi" % package : dburi}
        })
    else:
        config.update({'global':
            {"sqlobject.dburi" : dburi}
        })

def commit_all():
    """Commits the transactions in all registered hubs (for this thread)."""
    for hub in hub_registry:
        hub.commit()

def rollback_all():
    """Rollback the transactions in all registered hubs (for this thread)."""
    for hub in hub_registry:
        hub.rollback()

def end_all():
    """End the transactions in all registered hubs (for this thread)."""
    for hub in hub_registry:
        hub.end()

[dispatch.generic(MultiorderGenericFunction)]
def run_with_transaction(func, *args, **kw):
    pass

[dispatch.generic(MultiorderGenericFunction)]
def restart_transaction(args):
    pass

def _use_sa(args=None):
    # check to see if sqlalchemy has been imported and configured
    return _engine is not None

# include "args" to avoid call being pre-cached
[run_with_transaction.when("not _use_sa(args)")]
def so_rwt(func, *args, **kw):
    log.debug("Starting SQLObject transaction")
    try:
        try:
            retval = func(*args, **kw)
            commit_all()
            return retval
        except cherrypy.HTTPRedirect:
            commit_all()
            raise
        except cherrypy.InternalRedirect:
            commit_all()
            raise
        except:
            # No need to "rollback" the sqlalchemy unit of work,
            # because nothing has hit the db yet.
            rollback_all()
            raise
    finally:
        end_all()

# include "args" to avoid call being pre-cached
[restart_transaction.when("not _use_sa(args)")]
def so_restart_transaction(args):
    #log.debug("ReStarting SQLObject transaction")
    # Disable for now for compatibility
    pass

def dispatch_exception(exception, args, kw):
    # errorhandling import here to avoid circular imports
    from turbogears.errorhandling import dispatch_error
    # Keep in mind func is not the real func but _expose
    real_func, accept, allow_json, controller = args[:4]
    args = args[4:]
    exc_type, exc_value, exc_trace = sys.exc_info()
    remove_keys(kw, ("tg_source", "tg_errors", "tg_exceptions"))
    try:
        output = dispatch_error(
            controller, real_func, None, exception, *args, **kw)
    except dispatch.NoApplicableMethods:
        raise exc_type, exc_value, exc_trace
    else:
        del exc_trace
        return output

# include "args" to avoid call being pre-cached
[run_with_transaction.when("_use_sa(args)")]
def sa_rwt(func, *args, **kw):
    log.debug("Starting SA transaction")
    request.sa_transaction = make_sa_transaction(session)
    try:
        try:
            retval = func(*args, **kw)
        except (cherrypy.HTTPRedirect, cherrypy.InternalRedirect):
            # If a redirect happens, commit and proceed with redirect
            if sa_transaction_active():
                log.debug('Redirect in active transaction - will commit now')
                if hasattr(session, 'commit'):
                    session.commit()
                else: # SA < 0.4
                    request.sa_transaction.commit()
            else:
                log.debug('Redirect in inactive transaction')
            raise
        except:
            # If any other exception happens, rollback and re-raise error
            if sa_transaction_active():
                log.debug('Error in active transaction - will rollback now')
                if hasattr(session, 'rollback'):
                    session.rollback()
                else: # SA < 0.4
                    request.sa_transaction.rollback()
            else:
                log.debug('Error in inactive transaction')
            raise
        # If the call was successful, commit and proceed
        if sa_transaction_active():
            log.debug('Transaction is still active - will commit now')
            try:
                session.commit()
            except AttributeError: # SA < 0.4
                request.sa_transaction.commit()
        else:
            log.debug('Transaction is already inactive')
    finally:
        log.debug('Ending SA transaction')
        session.close()
    return retval

# include "args" to avoid call being pre-cached
[restart_transaction.when("_use_sa(args)")]
def sa_restart_transaction(args):
    log.debug("Restarting SA transaction")
    if sa_transaction_active():
        log.debug('Transaction is still active - will rollback now')
        try:
            session.rollback()
        except AttributeError: # SA < 0.4
            request.sa_transaction.rollback()
    else:
        log.debug('Transaction is already inactive')
    session.close()
    request.sa_transaction = make_sa_transaction(session)

def make_sa_transaction(session):
    """Create a new transaction in an SA session."""
    try:
        return session.begin()
    except AttributeError: # SA < 0.4
        return session.create_transaction()

def sa_transaction_active():
    """Check whether SA transaction is still active."""
    try:
        return session.is_active
    except AttributeError: # SA < 0.4.9
        try:
            return session().is_active
        except (TypeError, AttributeError): # SA < 0.4.7
            try:
                transaction = request.sa_transaction
            except AttributeError:
                return False
            try:
                return transaction and transaction.is_active
            except AttributeError: # SA < 0.4.3
                return transaction.session.transaction

def so_to_dict(sqlobj):
    """Convert SQLObject to a dictionary based on columns."""
    d = {}
    if sqlobj is None:
        return d # stops recursion
    for name in sqlobj.sqlmeta.columns.keys():
        d[name] = getattr(sqlobj, name)
    d['id'] = sqlobj.id # id must be added explicitly
    if sqlobj._inheritable:
        d.update(so_to_dict(sqlobj._parent))
        d.pop('childName')
    return d

def so_columns(sqlclass, columns=None):
    """Return a dict with all columns from a SQLObject.

    This includes the columns from InheritableSO's bases.

    """
    if columns is None:
        columns = {}
    columns.update(filter(lambda i: i[0] != 'childName',
                          sqlclass.sqlmeta.columns.items()))
    if sqlclass._inheritable:
        so_columns(sqlclass.__base__, columns)
    return columns

def so_joins(sqlclass, joins=None):
    """Return a list with all joins from a SQLObject.

    The list includes the columns from InheritableSO's bases.

    """
    if joins is None:
        joins = []
    joins.extend(sqlclass.sqlmeta.joins)
    if sqlclass._inheritable:
        so_joins(sqlclass.__base__, joins)
    return joins

class EndTransactionsFilter(BaseFilter):
    def on_end_resource(self):
        if _use_sa():
            try:
                session.expunge_all()
            except AttributeError: # SQLAlchemy < 0.5.1
                session.clear()
        end_all()

__all__ = ["get_engine", "metadata", "session", "mapper",
           "PackageHub", "AutoConnectHub", "set_db_uri",
           "commit_all", "rollback_all", "end_all", "so_to_dict",
           "so_columns", "so_joins", "EndTransactionsFilter"]



"""Graphical user interface for managing TurboGears projects"""

import pkg_resources
import cherrypy
import turbogears
from turbogears.util import get_project_name, setlike
from turbogears import controllers, expose


class Info(controllers.Controller):
    """TurboGears System Information.

    Lists your TurboGears packages and version information.

    """

    __label__ = "System Info"
    __version__ = "0.1"
    __author__ = "Ronald Jaramillo"
    __email__ = "ronald@checkandshare.com"
    __copyright__ = "Copyright 2005 Ronald Jaramillo"
    __license__ = "MIT"

    icon = "/tg_static/images/info.png"

    [expose(template='turbogears.toolbox.info')]
    def index(self):
        from turbogears.command.info import retrieve_info
        packages, plugins = retrieve_info(with_links=True)
        return dict(packages=packages, plugins=plugins)


class WidgetBrowser(controllers.Controller):
    """The widget browser.

    Browse usage samples, description and source code for the available
    TurboGears Widgets.

    """

    __label__ = "Widget Browser"
    __version__ = "0.1"
    __author__ = "Kevin Dangoor"
    __email__ = "dangoor+turbogears@gmail.com"
    __copyright__ = "Copyright 2005 Kevin Dangoor"
    __license__ = "MIT"

    all_descs = None
    icon = "/tg_static/images/widgets.png"

    [expose(template="turbogears.toolbox.widgets")]
    def index(self, name=None):
        from turbogears import widgets
        from turbogears.widgets import js_location, Tabber, SyntaxHighlighter
        all_descs = self.all_descs
        if not all_descs:
            widgets.load_widgets()
            all_descs = dict()
            for widgetdesc in widgets.all_widgets:
                wd = widgetdesc()
                all_descs[wd.full_class_name.replace('.', '_')] = wd
            self.all_descs = all_descs
        if name:
            all_descs = {name: all_descs[name]}
        desclist = all_descs.values()
        desclist.sort(cmp=lambda x, y: cmp(x.name.lower(), y.name.lower()))
        output = dict(descs=desclist, viewing_one=name != None)
        if name:
            # do not extend desclist!
            desclist = desclist + [Tabber(), SyntaxHighlighter()]

        css = setlike()
        js = dict()
        for location in js_location:
            js[location] = setlike()

        for widgetdesc in desclist:
            if not name and widgetdesc.show_separately:
                continue
            css.add_all(widgetdesc.retrieve_css())
            for script in widgetdesc.retrieve_javascript():
                js[getattr(script, 'location', js_location.head)].add(script)

        output['widget_css'] = css
        for location in js:
            output['widget_js_%s' % str(location)] = js[location]

        return output

    def __getattr__(self, widgetname):
        try:
            return self.all_descs[widgetname]
        except:
            raise AttributeError(widgetname)


class Toolbox(controllers.RootController):

    def __init__(self):
        self.toolbox = self.get_tools()

    def tool_icon(self, tool):
        icon = getattr(tool, 'icon', '')
        if icon:
            return icon

    def get_tools(self):
        project_name = get_project_name()
        toolbox = []
        for i in pkg_resources.iter_entry_points("turbogears.toolboxcommand"):
            tool = i.load()
            args = {
                'path': i.name,
                'label': getattr(tool, '__label__', i.name),
                'description': getattr(tool, '__doc__', ''),
                'version': getattr(tool, '__version__', ''),
                'author': getattr(tool, '__author__', ''),
                'email': getattr(tool, '__email__', ''),
                'copyright': getattr(tool, '__copyright__', ''),
                'license': getattr(tool, '__license__', ''),
                'icon': self.tool_icon(tool),
                'disabled': False}
            if project_name or not getattr(tool, 'need_project', False):
                try:
                    setattr(self, i.name, tool())
                except Exception, e:
                    args["description"] = str(e) or 'Tool could not be loaded.'
                    args["disabled"] = 'disabled'
            else:
                args["description"] += '\nNeeds project.'
                args["disabled"] = 'disabled'
            toolbox.append(args)
        return toolbox

    def arrange_in_pairs(self, tools):
        p = [[], []]
        for idx, tool in enumerate(tools):
            p[idx%2].append(tool)
        pairs = zip(p[0], p[1])
        if len(p[0]) > len(p[1]):
            pairs.append([p[0][-1], None])
        if len(p[0]) < len(p[1]):
            pairs.append([p[1][-1], None])
        return pairs

    [expose(template="turbogears.toolbox.main")]
    def index(self):
        return dict(toolbox = self.arrange_in_pairs(self.toolbox),
            project = get_project_name())

    [expose()]
    def noaccess(self):
        return """<h3>No access for %s</h3>
            <p>
               By default only localhost (127.0.0.1)
               has access to the Toolbox
            </p>
            <p>
               You can provide access to your client by passing
               your host address to Toolbox as a parameter. Ex:
            </p>
            <pre>
                tg-admin toolbox -c %s
            </pre>
            """ % (cherrypy.request.remoteAddr,cherrypy.request.remoteAddr)



from datetime import datetime

from sqlobject import SQLObject, SQLObjectNotFound, RelatedJoin, \
    DateTimeCol, IntCol, StringCol, UnicodeCol
from sqlobject.inheritance import InheritableSQLObject

import warnings
import logging
log = logging.getLogger("turbogears.identity.soprovider")

import turbogears
from turbogears import identity
from turbogears.database import PackageHub
from turbogears.util import load_class
from turbojson.jsonify import jsonify_sqlobject, jsonify

hub = PackageHub("turbogears.identity")
__connection__ = hub

try:
    set, frozenset
except NameError:  # Python 2.3
    from sets import Set as set, ImmutableSet as frozenset


def to_db_encoding(s, encoding):
    if isinstance(s, str):
        pass
    elif hasattr(s, '__unicode__'):
        s = unicode(s)
    if isinstance(s, unicode):
        s = s.encode(encoding)
    return s


class DeprecatedAttr(object):
    def __init__(self, old_name, new_name):
        self.old_name = old_name
        self.new_name = new_name

    def __get__(self, obj, type=None):
        warnings.warn("%s has been deprecated in favor of %s" %
            (self.old_name, self.new_name), DeprecationWarning)
        return getattr(obj, self.new_name)

    def __set__(self, obj, value):
        warnings.warn( "%s has been deprecated in favor of %s" %
            (self.old_name, self.new_name), DeprecationWarning)
        return setattr(obj, self.new_name, value)


# Global class references --
# these will be set when the provider is initialized.
user_class = None
group_class = None
permission_class = None
visit_class = None


class SqlObjectIdentity(object):
    """Identity that uses a model from a database (via SQLObject)."""

    def __init__(self, visit_key=None, user=None):
        self.visit_key = visit_key
        if user:
            self._user = user
            if visit_key is not None:
                self.login()

    def _get_user(self):
        """Get user instance for this identity."""
        try:
            return self._user
        except AttributeError:
            # User hasn't already been set
            pass
        # Attempt to load the user. After this code executes, there *will* be
        # a _user attribute, even if the value is None.
        visit = self.visit_link
        if visit:
            try:
                self._user = user_class.get(visit.user_id)
            except SQLObjectNotFound:
                log.warning("No such user with ID: %s", visit.user_id)
                self._user = None
        else:
            self._user = None
        return self._user
    user = property(_get_user)

    def _get_user_name(self):
        """Get user name of this identity."""
        if not self.user:
            return None
        return self.user.user_name
    user_name = property(_get_user_name)

    def _get_user_id(self):
        """Get user id of this identity."""
        if not self.user:
            return None
        return self.user.id
    user_id = property(_get_user_id)

    def _get_anonymous(self):
        """Return true if not logged in."""
        return not self.user
    anonymous = property(_get_anonymous)

    def _get_permissions(self):
        """Get set of permission names of this identity."""
        try:
            return self._permissions
        except AttributeError:
            # Permissions haven't been computed yet
            pass
        if not self.user:
            self._permissions = frozenset()
        else:
            self._permissions = frozenset(
                [p.permission_name for p in self.user.permissions])
        return self._permissions
    permissions = property(_get_permissions)

    def _get_groups(self):
        """Get set of group names of this identity."""
        try:
            return self._groups
        except AttributeError:
            # Groups haven't been computed yet
            pass
        if not self.user:
            self._groups = frozenset()
        else:
            self._groups = frozenset([g.group_name for g in self.user.groups])
        return self._groups
    groups = property(_get_groups)

    def _get_group_ids(self):
        """Get set of group IDs of this identity."""
        try:
            return self._group_ids
        except AttributeError:
            # Groups haven't been computed yet
            pass
        if not self.user:
            self._group_ids = frozenset()
        else:
            self._group_ids = frozenset([g.id for g in self.user.groups])
        return self._group_ids
    group_ids = property(_get_group_ids)

    def _get_visit_link(self):
        """Get the visit link to this identity."""
        if self.visit_key is None:
            return None
        try:
            return visit_class.by_visit_key(self.visit_key)
        except SQLObjectNotFound:
            return None
    visit_link = property(_get_visit_link)

    def _get_login_url(self):
        """Get the URL for the login page."""
        return identity.get_failure_url()
    login_url = property(_get_login_url)

    def login(self):
        """Set the link between this identity and the visit."""
        visit = self.visit_link
        if visit:
            visit.user_id = self._user.id
        else:
            visit = visit_class(visit_key=self.visit_key, user_id=self._user.id)

    def logout(self):
        """Remove the link between this identity and the visit."""
        visit = self.visit_link
        if visit:
            visit.destroySelf()
        # Clear the current identity
        identity.set_current_identity(SqlObjectIdentity())


class SqlObjectIdentityProvider(object):
    """IdentityProvider that uses a model from a database (via SQLObject)."""

    def __init__(self):
        super(SqlObjectIdentityProvider, self).__init__()
        get = turbogears.config.get

        global user_class, group_class, permission_class, visit_class

        user_class_path = get("identity.soprovider.model.user",
            __name__ + ".TG_User")
        user_class = load_class(user_class_path)
        if user_class:
            log.info("Successfully loaded \"%s\"" % user_class_path)
        try:
            self.user_class_db_encoding = \
                user_class.sqlmeta.columns['user_name'].dbEncoding
        except (KeyError, AttributeError):
            self.user_class_db_encoding = 'UTF-8'
        group_class_path = get("identity.soprovider.model.group",
            __name__ + ".TG_Group")
        group_class = load_class(group_class_path)
        if group_class:
            log.info("Successfully loaded \"%s\"" % group_class_path)

        permission_class_path = get("identity.soprovider.model.permission",
            __name__ + ".TG_Permission")
        permission_class = load_class(permission_class_path)
        if permission_class:
            log.info("Successfully loaded \"%s\"" % permission_class_path)

        visit_class_path = get("identity.soprovider.model.visit",
            __name__ + ".TG_VisitIdentity")
        visit_class = load_class(visit_class_path)
        if visit_class:
            log.info("Successfully loaded \"%s\"" % visit_class_path)

        # Default encryption algorithm is to use plain text passwords
        algorithm = get("identity.soprovider.encryption_algorithm", None)
        self.encrypt_password = lambda pw: \
            identity._encrypt_password(algorithm, pw)

    def create_provider_model(self):
        """Create the database tables if they don't already exist."""
        try:
            hub.begin()
            user_class.createTable(ifNotExists=True)
            group_class.createTable(ifNotExists=True)
            permission_class.createTable(ifNotExists=True)
            visit_class.createTable(ifNotExists=True)
            hub.commit()
            hub.end()
        except KeyError:
            log.warning("No database is configured:"
                " SqlObjectIdentityProvider is disabled.")
            return

    def validate_identity(self, user_name, password, visit_key):
        """Validate the identity represented by user_name using the password.

        Must return either None if the credentials weren't valid or an object
        with the following properties:
            user_name: original user name
            user: a provider dependent object (TG_User or similar)
            groups: a set of group names
            permissions: a set of permission names

        """
        try:
            user_name = to_db_encoding(user_name, self.user_class_db_encoding)
            user = user_class.by_user_name(user_name)
            if not self.validate_password(user, user_name, password):
                log.info("Passwords don't match for user: %s", user_name)
                return None
            log.info("Associating user (%s) with visit (%s)",
                user_name, visit_key)
            return SqlObjectIdentity(visit_key, user)
        except SQLObjectNotFound:
            log.warning("No such user: %s", user_name)
            return None

    def validate_password(self, user, user_name, password):
        """Check the user_name and password against existing credentials.

        Note: user_name is not used here, but is required by external
        password validation schemes that might override this method.
        If you use SqlObjectIdentityProvider, but want to check the passwords
        against an external source (i.e. PAM, a password file, Windows domain),
        subclass SqlObjectIdentityProvider, and override this method.

        """
        return user.password == self.encrypt_password(password)

    def load_identity(self, visit_key):
        """Lookup the principal represented by user_name.

        Return None if there is no principal for the given user ID.

        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependent object (TG_User or similar)
            groups: a set of group names
            permissions: a set of permission names

        """
        return SqlObjectIdentity(visit_key)

    def anonymous_identity(self):
        """Return anonymous identity.

        Must return an object with the following properties:
            user_name: original user name
            user: a provider dependent object (TG_User or similar)
            groups: a set of group names
            permissions: a set of permission names

        """
        return SqlObjectIdentity()

    def authenticated_identity(self, user):
        """Constructs Identity object for users with no visit_key."""
        return SqlObjectIdentity(user=user)


class TG_VisitIdentity(SQLObject):
    """A visit to your website."""
    class sqlmeta:
        table = "tg_visit_identity"

    visit_key = StringCol(length=40, alternateID=True,
        alternateMethodName="by_visit_key")
    user_id = IntCol()


class TG_Group(InheritableSQLObject):
    """An ultra-simple group definition."""
    class sqlmeta:
        table = "tg_group"

    group_name = UnicodeCol(length=16, alternateID=True,
        alternateMethodName="by_group_name")
    display_name = UnicodeCol(length=255)
    created = DateTimeCol(default=datetime.now)

    # Old names
    groupId = DeprecatedAttr("groupId", "group_name")
    displayName = DeprecatedAttr("displayName", "display_name")

    # collection of all users belonging to this group
    users = RelatedJoin("TG_User", intermediateTable="tg_user_group",
        joinColumn="group_id", otherColumn="user_id")

    # collection of all permissions for this group
    permissions = RelatedJoin("TG_Permission", joinColumn="group_id",
        intermediateTable="tg_group_permission",
        otherColumn="permission_id")

[jsonify.when('isinstance(obj, TG_Group)')]
def jsonify_group(obj):
    """Convert group to JSON."""
    result = jsonify_sqlobject(obj)
    result["users"] = [u.user_name for u in obj.users]
    result["permissions"] = [p.permission_name for p in obj.permissions]
    return result


class TG_User(InheritableSQLObject):
    """Reasonably basic User definition."""
    class sqlmeta:
        table = "tg_user"

    user_name = UnicodeCol(length=16, alternateID=True,
        alternateMethodName="by_user_name")
    email_address = UnicodeCol(length=255, alternateID=True,
        alternateMethodName="by_email_address")
    display_name = UnicodeCol(length=255)
    password = UnicodeCol(length=40)
    created = DateTimeCol(default=datetime.now)

    # Old attribute names
    userId = DeprecatedAttr("userId", "user_name")
    emailAddress = DeprecatedAttr("emailAddress", "email_address")
    displayName = DeprecatedAttr("displayName", "display_name")

    # groups this user belongs to
    groups = RelatedJoin("TG_Group", intermediateTable="tg_user_group",
        joinColumn="user_id", otherColumn="group_id")

    def _get_permissions(self):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms

    def _set_password(self, cleartext_password):
        """Run cleartext_password through the hash algorithm before saving."""
        try:
            hash = identity.current_provider.encrypt_password(cleartext_password)
        except identity.exceptions.IdentityManagementNotEnabledException:
            # Creating identity provider just to encrypt password
            # (so we don't reimplement the encryption step).
            ip = SqlObjectIdentityProvider()
            hash = ip.encrypt_password(cleartext_password)
            if hash == cleartext_password:
                log.info("Identity provider not enabled,"
                    " and no encryption algorithm specified in config."
                    " Setting password as plaintext.")
        self._SO_set_password(hash)

    def set_password_raw(self, password):
        """Save the password as-is to the database."""
        self._SO_set_password(password)

[jsonify.when('isinstance(obj, TG_User)')]
def jsonify_user(obj):
    """Convert user to JSON."""
    result = jsonify_sqlobject(obj)
    del result['password']
    result["groups"] = [g.group_name for g in obj.groups]
    result["permissions"] = [p.permission_name for p in obj.permissions]
    return result


class TG_Permission(InheritableSQLObject):
    """Permissions for a given group."""
    class sqlmeta:
        table = "tg_permission"

    permission_name = UnicodeCol(length=16, alternateID=True,
        alternateMethodName="by_permission_name")
    description = UnicodeCol(length=255)

    # Old attributes
    permissionId = DeprecatedAttr("permissionId", "permission_name")

    groups = RelatedJoin("TG_Group", intermediateTable="tg_group_permission",
        joinColumn="permission_id", otherColumn="group_id")

[jsonify.when('isinstance(obj, TG_Permission)')]
def jsonify_permission(obj):
    """Convert permissions to JSON."""
    result = jsonify_sqlobject(obj)
    result["groups"] = [g.group_name for g in obj.groups]
    return result


def encrypt_password(cleartext_password):
    """Encrypt given cleartext password."""
    try:
        hash = identity.current_provider.encrypt_password(cleartext_password)
    except identity.exceptions.RequestRequiredException:
        # Creating identity provider just to encrypt password
        # (so we don't reimplement the encryption step).
        ip = SqlObjectIdentityProvider()
        hash = ip.encrypt_password(cleartext_password)
        if hash == cleartext_password:
            log.info("Identity provider not enabled, and no encryption "
                "algorithm specified in config. Setting password as plaintext.")
    return hash



import unittest

import cherrypy
from turbogears import controllers, expose, redirect, testutil

class MyRoot(controllers.RootController):
    def set_name(self, name):
        cookies = cherrypy.response.simple_cookie
        cookies['name'] = name
        return "Hello " + name
    set_name = expose()(set_name)

    def get_name(self):
        cookies = cherrypy.request.simple_cookie
        if 'name' in cookies:
            return cookies['name'].value
        else:
            return "cookie not found"
    get_name = expose()(get_name)

    def get_unicode_name(self):
        """Return a nonsense string of non-ascii characters"""
        cherrypy.response.headers['Content-Type'] = 'text/plain; encoding=utf-8'
        return u'\u1234\u9876\u3456'.encode('utf-8')
    get_unicode_name = expose()(get_unicode_name)

    def redirect(self):
        raise redirect("foo")
    redirect = expose()(redirect)

class TestControllers(unittest.TestCase):
    def setUp(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}
        cherrypy.tree.mount(MyRoot(), "/")
        self.root = cherrypy.root

    def tearDown(self):
        cherrypy.root = None
        cherrypy.tree.mount_points = {}

    def test_browser_session(self):
        bs = testutil.BrowsingSession()
        bs.goto('/get_name')
        assert bs.response == 'cookie not found'
        bs.goto('/set_name?name=me')
        bs.goto('/get_name')
        assert bs.response == 'me'

    def test_browser_session_for_two_users(self):
        bs1 = testutil.BrowsingSession()
        bs2 = testutil.BrowsingSession()
        bs1.goto('/set_name?name=bs1')
        bs2.goto('/set_name?name=bs2')
        bs1.goto('/get_name')
        assert bs1.response == 'bs1'
        bs2.goto('/get_name')
        assert bs2.response == 'bs2'

    def test_unicode_response(self):
        bs = testutil.BrowsingSession()
        bs.goto('/get_unicode_name')
        assert bs.response == u'\u1234\u9876\u3456'.encode('utf-8')
        assert bs.unicode_response == u'\u1234\u9876\u3456'
        assert type(bs.unicode_response) == unicode

    def test_index(self):
        "Test that call_with_request is usable if controller uses redirect"
        try:
            testutil.call(self.root.redirect)
            self.fail("no redirect exception raised")
        except cherrypy.HTTPRedirect, e:
            assert e.status in [302, 303]
            self.assertEquals(1, len(e.urls))
            assert e.urls[0].endswith("foo")



# test-file: test_command_i18n.py

"""Command-line user interface for i18n administration."""

import re
import glob
import os
import os.path
import atexit
import optparse
import tempfile

from pkg_resources import resource_filename
import formencode
import kid

import turbogears
import turbogears.i18n
from turbogears import config
from turbogears.toolbox.admi18n import pygettext, msgfmt, catalog
from turbogears.toolbox.admi18n.catalog import normalize
from turbogears.command.base import silent_os_remove
from turbogears.util import load_project_config, get_package_name


class ProgramError(StandardError):
    """Signals about a general application error."""


def copy_file(src, dest):
    if os.path.exists(dest):
        os.remove(dest)
    fh = open(src, 'rb')
    data = fh.read()
    fh.close()
    fh = open(dest, 'wb')
    fh.write(data)
    fh.close()

_str_literal = r"""(?:'((?:[^']|\\')*)'|"((?:[^"]|\\")*)")"""
_py_i18n_re = re.compile(r"\b_\s*\(\s*[uU]?[rR]?%s\s*\)" % _str_literal)
_js_i18n_re = re.compile(r"\b_\s*\(\s*%s\s*\)" % _str_literal)


class InternationalizationTool(object):
    """Manages i18n data via command-line interface.

    Contributed to TurboGears by Max Ischenko (http://maxischenko.in.ua).

    """

    desc = "Manage i18n data"
    need_project = True
    config = None
    load_config = True
    locale_dir = 'locales'
    domain = 'messages'

    name = None
    package = None

    def __init__(self, version):
        parser = optparse.OptionParser(usage="""
%prog [options] <command>

Available commands:
  add <locale>        Creates a message catalog for specified locale
  collect             Scan source files to gather translatable strings in a .pot file
  merge               Sync message catalog in different languages with .pot file
  compile             Compile message catalog (.po -> .mo)
  create_js_messages  Create message catalogs for JS usage
  clean               Delete backups and compiled files
""", version="%prog " + version)
        parser.add_option("-f", "--force", default=False,
            action="store_true", dest="force_ops",
            help="Force potentially damaging actions")
        parser.add_option("-a", "--ascii", default=False,
            action="store_true", dest="ascii_output",
            help="Escape non-ascii characters (.py files only)")
        parser.add_option("-K", "--no-kid-support", default=True,
            action="store_false", dest="kid_support",
            help="Do not extract messages from Kid templates")
        parser.add_option("", "--src-dir", default=None,
            action="store", dest="source_dir",
            help="Directory that contains source files")
        parser.add_option("", "--no-js-support", default=True,
            action="store_false", dest="js_support",
            help="Extract messages from js-files.")
        parser.add_option("", "--js-base-dir",
            action="store", dest="js_base_dir",
            default="static/javascript",
            help="Base directory of javascript files"
                " for generated message-files.")
        parser.add_option("-e", "--js-encoding",
            dest="js_encoding", default="utf-8", metavar="ENCODING",
            help="Encoding of JavaScript source files (default: UTF-8)")
        parser.set_defaults(js_support=True)
        self.parser = parser

    def load_project_config(self):
        """Choose the config file.

        Try to guess whether this is a development or installed project.

        """

        # check whether user specified custom settings
        if self.load_config:
            load_project_config(self.config)

        if config.get("i18n.locale_dir"):
            self.locale_dir = config.get("i18n.locale_dir")
            print 'Use %s as a locale directory' % self.locale_dir
        if config.get('i18n.domain'):
            self.domain = config.get("i18n.domain")
            print 'Use %s as a message domain' % self.domain

        if os.path.exists(self.locale_dir) and \
                not os.path.isdir(self.locale_dir):
            raise ProgramError, (
                '%s is not a directory' % self.locale_dir)

        if not os.path.exists(self.locale_dir):
            os.makedirs(self.locale_dir)

    def parse_args(self):
        return self.parser.parse_args()

    def run(self):
        self.load_project_config()
        options, args = self.parse_args()
        if not args:
            self.parser.error("No command specified")
        self.options = options
        command, args = args[0], args[1:]
        if 'collect' == command:
            self.scan_source_files()
        elif 'add' == command:
            self.add_languages(args)
        elif 'compile' == command:
            self.compile_message_catalogs()
        elif 'merge' == command:
            self.merge_message_catalogs()
        elif 'clean' == command:
            self.clean_generated_files()
        elif 'create_js_messages' == command:
            self.create_js_messages()
        else:
            self.parser.error("Command not recognized")

    def create_js_messages(self):
        self.load_project_config()
        languages = []
        # we assume the the structure of messages is always
        # <self.locale_dir>/<lang>/LC_MESSAGES ...
        # to extract the languages known to the app
        locale_dir_prefix = self.locale_dir.split(os.sep)
        for fname in self.list_message_catalogs():
            languages.append(fname.split(os.sep)[len(locale_dir_prefix):][0])
        import turbogears.i18n.utils as utils
        srcdir = self.options.source_dir or get_package_name().split('.', 1)[0]
        def list_js_files():
            for root, dirs, files in os.walk(srcdir):
                if os.path.basename(root).lower() in ('cvs', '.svn'):
                    continue
                for fname in files:
                    name, ext = os.path.splitext(fname)
                    srcfile = os.path.join(root, fname)
                    if ext == '.js':
                        yield srcfile
        def escape(arg):
            if "'" in arg:
                return '"%s"' % arg
            return "'%s'" % arg
        for locale in languages:
            def gl():
                return locale
            utils._get_locale = gl
            messages = []
            for filename in list_js_files():
                for key in self.get_strings_in_js(os.path.join(filename))[0]:
                    key = unicode(key)
                    msg = unicode(_(key, locale))
                    messages.append((key, msg))
            # for a final return
            header = """
if (typeof(MESSAGES) == "undefined") {
    MESSAGES = {};
}

LANG = '%s';
_messages = [
""" % locale
            footer = """
             ];

for(var i in _messages) {
  MESSAGES[_messages[i][0]] = _messages[i][1];
 }
"""
            message_block = u",\n".join(["[%s , %s]" % (escape(msgid),
                escape(msgstr)) for msgid, msgstr in messages]).encode("utf-8")
            message_block = message_block + "\n"
            outfilename = os.path.join(srcdir, self.options.js_base_dir,
                    'messages-%s.js' % locale)
            print "Creating message file <%s>." % outfilename
            mf = open(outfilename, "w")
            mf.write(header)
            mf.write(message_block)
            mf.write(footer)
            mf.close()

    def clean_generated_files(self):
        potfile = self.get_potfile_path()
        silent_os_remove(potfile.replace('.pot', '.bak'))
        for fname in self.list_message_catalogs():
            silent_os_remove(fname.replace('.po', '.mo'))
            silent_os_remove(fname.replace('.po', '.back'))

    def merge_message_catalogs(self):
        potfile = self.get_potfile_path()
        catalogs = self.list_message_catalogs()
        catalog.merge(potfile, catalogs)

    def compile_message_catalogs(self):
        for fname in self.list_message_catalogs():
            dest = fname.replace('.po','.mo')
            msgfmt.make(fname, dest)
            if os.path.exists(dest):
                print 'Compiled %s OK' % fname
            else:
                print 'Compilation of %s failed!' % fname

    def _copy_file_withcheck(self, sourcefile, targetfile):
        if not (os.path.exists(targetfile) and not self.options.force_ops):
            copy_file(sourcefile, targetfile)
            print 'Copy', sourcefile, 'to', targetfile
        else:
            print "File %s exists, use --force to override" % targetfile

    def _copy_moduletranslation(self, sourcefile, targetdir, language):
        modulefilename = os.path.basename(sourcefile)
        if os.path.exists(sourcefile):
            targetfile = os.path.join(targetdir, modulefilename)
            self._copy_file_withcheck(sourcefile, targetfile)
        else:
            print ("%s translation for language '%s' does not exist"
                " (file searched '%s').\nPlease consider to contribute"
                " a translation." % (modulefilename, language, sourcefile))

    def add_languages(self, codes):
        potfile = self.get_potfile_path()
        if not os.path.isfile(potfile):
            print "Run 'collect' first to create", potfile
            return
        for code in codes:
            catalog_file = self.get_locale_catalog(code)
            langdir = os.path.dirname(catalog_file)
            if not os.path.exists(langdir):
                os.makedirs(langdir)

            sourcefile_fe = os.path.join(formencode.api.get_localedir(), code, \
                "LC_MESSAGES","FormEncode.mo")
            self._copy_moduletranslation(sourcefile_fe, langdir, code)

            basedir_i18n_tg = resource_filename("turbogears.i18n", "data")
            sourcefile_tg  = os.path.join(basedir_i18n_tg, code, \
                                          "LC_MESSAGES", "TurboGears.mo")
            self._copy_moduletranslation(sourcefile_tg, langdir, code)

            self._copy_file_withcheck(potfile, catalog_file)

    def scan_source_files(self):
        source_files = []
        kid_files = []
        js_files = []
        srcdir = self.options.source_dir or get_package_name().split('.', 1)[0]
        print 'Scanning source directory', srcdir
        for root, dirs, files in os.walk(srcdir):
            if os.path.basename(root).lower() in ('cvs', '.svn'):
                continue
            for fname in files:
                name, ext = os.path.splitext(fname)
                srcfile = os.path.join(root, fname)
                if ext == '.py':
                    source_files.append(srcfile)
                elif ext == '.kid':
                    kid_files.append(srcfile)
                elif ext == '.js':
                    js_files.append(srcfile)
                else:
                    pass # do nothing
        tmp_handle, tmp_potfile = tempfile.mkstemp(
            '.pot', 'tmp', self.locale_dir)
        os.close(tmp_handle)
        potbasename = os.path.basename(tmp_potfile)[:-4]
        pygettext_options = ['-v', '-d', potbasename, \
                '-p', os.path.dirname(tmp_potfile)]
        if self.options.ascii_output:
            pygettext_options.insert(0, '-E')
        pygettext.sys.argv = [''] + pygettext_options + source_files
        pygettext.main()
        if not os.path.exists(tmp_potfile):
            raise ProgramError, 'pygettext failed'
        atexit.register(silent_os_remove, tmp_potfile)
        if kid_files and self.options.kid_support:
            self.scan_kid_files(tmp_potfile, kid_files)
        if js_files and self.options.js_support:
            self.scan_js_files(tmp_potfile, js_files)
        potfile = self.get_potfile_path()
        if os.path.isfile(potfile):
            bakfile = potfile.replace('.pot', '.bak')
            silent_os_remove(bakfile)
            os.rename(potfile, bakfile)
            print 'Backup existing file to', bakfile
        os.rename(tmp_potfile, potfile)
        print 'Message templates written to', potfile

    def scan_kid_files(self, potfile, files):
        messages = []
        tags_to_ignore = ['script', 'style']
        keys = []

        def process_text(is_attribute, k, tag):
            key = None
            s = _py_i18n_re.search(k)
            if s:
                key = (s.group(1) or s.group(2) or '').strip()
            elif not is_attribute:
                # we don't have a kid expression in there, so it is
                # "just" a text entry - which we want to be translated!
                import kid.codewriter as cw
                parts = cw.interpolate(k)
                if isinstance(parts, list) and len(parts) > 1:
                    print "Warning: Mixed content in tag <%s>: %s" % (tag, k)
                elif isinstance(parts, basestring):
                    key = k.strip()
            if key and key not in keys and tag not in tags_to_ignore:
                messages.append((tag, fname, key))
                keys.append(key)

        for fname in files:
            print 'Working on', fname
            tree = None
            fh = open(fname)
            try:
                tree = kid.document(fh)
            except Exception, e:
                fh.close()
                print 'Skip %s: %s' % (fname, e)
                continue
            sentinel = None
            tag = None
            for ev, el in tree:
                if ev == kid.parser.START:
                    if not isinstance(el.tag, unicode):
                        # skip comments, processing instructions etc.
                        continue
                    if el.get('lang', None) is not None:
                        # if we have a lang-attribute, ignore this
                        # node AND all it's descendants.
                        sentinel = el
                        continue
                    # set the tag from the current one.
                    tag = re.sub('({[^}]+})?(\w+)', '\\2', el.tag)
                    if tag in ('script', 'style'):
                        # skip JavaScript, CSS etc.
                        sentinel = el
                        continue
                    # process the attribute texts
                    for attrib_text in el.attrib.values():
                        process_text(True, attrib_text, tag)
                elif ev == kid.parser.END:
                    if el is sentinel:
                        sentinel = None
                elif ev == kid.parser.TEXT:
                    if sentinel is None and el.strip():
                        process_text(False, el, tag)
            fh.close()
        fd = open(potfile, 'at+')
        for tag, fname, text in messages:
            text = normalize(text.encode('utf-8'))
            print >> fd, '#: %s:%s' % (fname, tag)
            print >> fd, 'msgid %s' % text
            print >> fd, 'msgstr ""'
            print >> fd, ''
        fd.close()

    def get_strings_in_js(self, fname):
        messages = []
        keys = []
        fh = open(fname)
        for i, line in enumerate(fh):
            s = _js_i18n_re.search(line.decode(self.options.js_encoding))
            while s:
                key = s.group(1) or s.group(2)
                pos = s.end()
                if key and key not in keys:
                    messages.append((i + 1, fname, key))
                    keys.append(key)
                s = _js_i18n_re.search(line, pos)
        fh.close()
        return keys, messages

    def scan_js_files(self, potfile, files):
        messages = []
        keys = []
        for fname in files:
            print 'Working on', fname
            k, m = self.get_strings_in_js(fname)
            keys.extend(k)
            messages.extend(m)
        self._write_potfile_entries(potfile, messages)

    def _write_potfile_entries(self, potfile, messages):
        if messages:
            fd = open(potfile, 'at+')
            for linenumber, fname, text in messages:
                if text == '':
                    continue
                text = catalog.normalize(text.encode('utf-8'))
                fd.write('#: %s:%s\n' % (fname, linenumber))
                fd.write('msgid %s\n' % text)
                fd.write('msgstr ""\n\n')
            fd.close()

    def get_potfile_path(self):
        return os.path.join(self.locale_dir, '%s.pot' % self.domain)

    def get_locale_catalog(self, code):
        return os.path.join(self.locale_dir, code, 'LC_MESSAGES',
            '%s.po' % self.domain)

    def list_message_catalogs(self):
        files = []
        for name in glob.glob(self.locale_dir + '/*'):
            if os.path.isdir(name):
                fname = os.path.join(name, 'LC_MESSAGES', '%s.po' % self.domain)
                if os.path.isfile(fname):
                    files.append(fname)
        return files

    def fix_tzinfo(self, potfile):
        """Fix tzinfo.

        In certain enviroments, tzinfo as formatted by strftime() is not utf-8,
        e.g. Windows XP with Russian MUL.

        This leads to later error when a program trying to read catalog.

        """
        fh = open(potfile, 'rb')
        data = fh.read()
        fh.close()
        def repl(m):
            """Remove tzinfo if it breaks encoding."""
            tzinfo = m.group(2)
            try:
                tzinfo.decode('utf-8')
            except UnicodeDecodeError:
                return m.group(1) # cut tz info
            return m.group(0) # leave unchanged
        data = re.sub(
            "(POT-Creation-Date: [\d-]+ [0-9:]+)\+([^\\\\]+)", repl, data)
        fh = open(potfile, 'wb')
        fh.write(data)
        fh.close()


def main():
    tool = InternationalizationTool()
    tool.run()


if __name__ == '__main__':
    main()



# Formatting configuration for locale fi

languages={'vi': 'vietnam', 'el': 'kreikka', 'eo': 'esperanto', 'en': 'englanti', 'zh': 'kiina', 'af': 'afrikanhollanti', 'sw': 'suahili', 'ca': 'katalaani', 'am': 'amharan kieli', 'gu': 'gujarati', 'sv': 'ruotsi', 'cs': 'tsekki', 'ar': 'arabia', 'mk': 'makedonia', 'ga': 'iiri', 'eu': 'baski', 'et': 'viro', 'az': 'azerbaizani', 'id': 'indonesia', 'es': 'espanja', 'ru': u'ven\xe4j\xe4', 'gl': 'galicia', 'nl': 'hollanti', 'nn': 'uusnorja', 'la': 'latina', 'nb': 'kirjanorja', 'tr': 'turkki', 'lv': 'latvia', 'lt': 'liettua', 'tk': 'tagalog', 'th': 'thai', 'ti': 'tigrinja', 'it': 'italia', 'so': 'somali', 'he': 'heprea', 'is': 'islanti', 'pl': 'puola', 'haw': 'havaiji', 'be': u'valkoven\xe4j\xe4', 'fr': 'ranska', 'bg': 'bulgaria', 'uk': 'ukraina', 'sl': 'sloveeni', 'hr': 'kroaatti', 'iu': 'eskimo', 'bn': 'bengali', 'de': 'saksa', 'bh': 'bihari', 'om': 'oromo', 'da': 'tanska', 'fa': 'farsi', 'hi': 'hindi', 'fi': 'suomi', 'hy': 'armenia', 'hu': 'unkari', 'ur': 'urdu', 'ja': 'japani', 'fo': u'f\xe4\xe4ri', 'ka': 'georgia', 'ro': 'romania', 'gv': 'manx', 'pt': 'portugali', 'uz': 'uzbekki', 'kk': 'kazakki', 'sr': 'serbia', 'sq': 'albania', 'no': 'norja', 'ko': 'korea', 'kn': 'kannada', 'km': 'khmer', 'kl': u'gr\xf6\xf6nlandi', 'sk': 'slovakki', 'mt': 'malta', 'sh': 'serbokroatia', 'kw': 'korni', 'ku': 'kurdi', 'mr': 'marathi', 'te': 'telugu', 'ta': 'tamili', 'my': 'burma'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia ja Hertsegovina', 'BB': 'Barbados', 'WF': 'Wallis ja Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': "Bouvet'nsaari", 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasilia', 'BS': 'Bahama', 'BY': u'Valko-Ven\xe4j\xe4', 'BZ': 'Belize', 'RU': u'Ven\xe4j\xe4', 'RW': 'Ruanda', 'TL': u'It\xe4-Timor', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tadzikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'Etel\xe4-Georgia ja Etel\xe4iset Sandwichsaaret', 'GR': 'Kreikka', 'GQ': u'P\xe4iv\xe4ntasaajan Guinea', 'GP': 'Guadeloupe', 'JP': 'Japani', 'GY': 'Guyana', 'GF': 'Ranskan Guayana', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'Iso-Britannia', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6nlanti', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Jordania', 'SP': 'Serbia', 'HR': 'Kroatia', 'HT': 'Haiti', 'HU': 'Unkari', 'HK': 'Hongkongin erityishallintoalue', 'HN': 'Honduras', 'HM': 'Heard ja McDonaldinsaaret', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': 'Palestiina', 'PW': 'Palau', 'PT': 'Portugali', 'SJ': 'Huippuvuoret ja Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': 'Ranskan Polynesia', 'PG': 'Papua-Uusi-Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filippiinit', 'PN': 'Pitcairn', 'PL': 'Puola', 'PM': 'Saint-Pierre ja Miquelon', 'ZM': 'Sambia', 'EH': u'L\xe4nsi-Sahara', 'EE': 'Viro', 'EG': 'Egypti', 'ZA': u'Etel\xe4-Afrikka', 'EC': 'Equador', 'IT': 'Italia', 'VN': 'Vietnam', 'SB': 'Salomonsaaret', 'ET': 'Etiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi-Arabia', 'ES': 'Espanja', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macaon erityishallintoalue', 'MN': 'Mongolia', 'MH': 'Marshallinsaaret', 'MK': 'Makedonia', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Malediivit', 'MQ': 'Martinique', 'MP': 'Pohjois-Mariaanit', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malesia', 'MX': 'Meksiko', 'IL': 'Israel', 'FR': 'Ranska', 'IO': u'Brittil\xe4inen Intian valtameren alue', 'SH': 'Saint Helena', 'FI': 'Suomi', 'FJ': 'Fidzi', 'FK': 'Falklandinsaaret', 'FM': 'Mikronesia', 'FO': u'F\xe4rsaaret', 'NI': 'Nicaragua', 'NL': 'Alankomaat', 'NO': 'Norja', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Uusi-Kaledonia', 'NE': 'Niger', 'NF': 'Norfolkinsaari', 'NG': 'Nigeria', 'NZ': 'Uusi Seelanti', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cookinsaaret', 'CI': 'Norsunluurannikko', 'CH': 'Sveitsi', 'CO': 'Kolumbia', 'CN': 'Kiina', 'CM': 'Kamerun', 'CL': 'Chile', 'CC': 'Kookossaaret', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Keski-Afrikan tasavalta', 'CD': 'Kongon demokraattinen tasavalta', 'CZ': 'Tsekin tasavalta', 'CY': 'Kypros', 'CX': 'Joulusaari', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Kuuba', 'SZ': 'Swazimaa', 'SY': 'Syyria', 'KG': 'Kirgisia', 'KE': 'Kenia', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kambodza', 'KN': 'Saint Kitts ja Nevis', 'KM': 'Komorit', 'ST': 'Sao Tome ja Principe', 'SK': 'Slovakia', 'KR': u'Etel\xe4-Korea', 'SI': 'Slovenia', 'KP': 'Pohjois-Korea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellit', 'KZ': 'Kazakstan', 'KY': 'Caymansaaret', 'SG': 'Singapore', 'SE': 'Ruotsi', 'SD': 'Sudan', 'DO': 'Dominikaaninen tasavalta', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Tanska', 'VG': u'Brittil\xe4iset Neitsytsaaret', 'DE': 'Saksa', 'YE': 'Jemen', 'DZ': 'Algeria', 'US': 'Yhdysvallat', 'UY': 'Uruguay', 'YU': 'Jugoslavia', 'YT': 'Mayotte', 'UM': 'Yhdysvaltain pienet erillissaaret', 'LB': 'Libanon', 'LC': 'Saint Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Kiinan maakunta Taiwan', 'TT': 'Trinidad ja Tobago', 'TR': 'Turkki', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Latvia', 'TO': 'Tonga', 'LT': 'Liettua', 'LU': 'Luxemburg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thaimaa', 'TF': u'Ranskan etel\xe4iset alueet', 'TG': 'Togo', 'TD': 'Tsad', 'TC': 'Turks- ja Caicossaaret', 'LY': 'Libya', 'VA': 'Vatikaani', 'VC': 'Saint Vincent ja Grenadiinit', 'AE': 'Arabiemiirikunnat', 'AD': 'Andorra', 'AG': 'Antigua ja Barbuda', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'Yhdysvaltain Neitsytsaaret', 'IS': 'Islanti', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Alankomaiden Antillit', 'AQ': 'Antarktis', 'AS': 'Amerikan Samoa', 'AR': 'Argentiina', 'AU': 'Australia', 'AT': u'It\xe4valta', 'AW': 'Aruba', 'IN': 'Intia', 'TZ': 'Tansania', 'AZ': 'Azerbaidzan', 'IE': 'Irlanti', 'ID': 'Indonesia', 'UA': 'Ukraina', 'QA': 'Qatar', 'MZ': 'Mosambik'}

months=['tammikuu', 'helmikuu', 'maaliskuu', 'huhtikuu', 'toukokuu', u'kes\xe4kuu', u'hein\xe4kuu', 'elokuu', 'syyskuu', 'lokakuu', 'marraskuu', 'joulukuu']

abbrMonths=['tammi', 'helmi', 'maalis', 'huhti', 'touko', u'kes\xe4', u'hein\xe4', 'elo', 'syys', 'loka', 'marras', 'joulu']

days=['maanantai', 'tiistai', 'keskiviikko', 'torstai', 'perjantai', 'lauantai', 'sunnuntai']

abbrDays=['ma', 'ti', 'ke', 'to', 'pe', 'la', 'su']

dateFormats={'medium': '%d.%m.%Y', 'full': '%d. %%(monthname)sta %Y', 'long': '%d. %%(monthname)sta %Y', 'short': '%d.%m.%Y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



import os
import codecs
from cherrypy import request
from turbogears.i18n import get_locale
from turbogears import startup, config
from turbogears.util import parse_http_accept_header
from turbogears.i18n.utils import lang_in_gettext_format
from turbogears.widgets.base import JSLink, CoreWD, RenderOnlyWD, static

__all__ = ["LocalizableJSLink", "CalendarLangFileLink"]


class LocalizableJSLink(JSLink):
    """Provides a simple way to include language-specific data in your JS code.

    Language file to use is determined from the user's locale or from the 'language'
    parameter. If there is no language file for the language (determined via
    'supported_languages' parameter) than 'default_language' is used.

    """
    default_language = 'en'
    supported_languages = ['en']
    params = ['default_language', 'language', 'supported_languages']
    params_doc = {
        'language' : 'language code to use ' \
            '(overrides user locale setting which is the default)',
        'default_language' : 'language code to use ' \
                'if specified language is not supported',
        'supported_languages' : 'list of supported language codes ' \
                ' (which means corresponding language files exist)',
    }

    def update_params(self, d):
        super(LocalizableJSLink, self).update_params(d)
        language = d.get('language') or get_locale()
        if language not in self.supported_languages:
            language = self.default_language
        d['link'] = d['link'] % {'lang':language}


class LocalizableJSLinkDesc(CoreWD, RenderOnlyWD):
    name = "Localizable JS Link"
    for_widget = LocalizableJSLink("turbogears", "js/yourscript-%(lang)s.js")


_norm_tag = ''.join([(c.islower() or c.isdigit() or c == '-')
    and c or c.isupper() and c.lower() or ' '
    for c in map(chr, xrange(256))])

def norm_tag(tag):
    """Normalize string to alphanumeric ascii chars, hyphens and underscores.

    The length is limited to 16 characters and other characters are
    collapsed to single underscore, or ignored at the start or end.

    """
    if tag is not None:
        return '_'.join(str(tag)[:16].translate(_norm_tag).split())


def norm_charset(charset):
    """Normalize the name of a character set."""
    if charset is not None:
        charset = norm_tag(charset)
        try:
            charset = norm_tag(codecs.lookup(charset).name)
        except (LookupError, AttributeError, ValueError):
            # AttributeError only needed for Py < 2.5
            # ValueError only needed for Py < 2.4
            pass
        return charset


class CalendarLangFileLink(LocalizableJSLink):
    """Links to proper calendar.js language file depending on HTTP info."""

    default_charset = 'utf-8'

    def update_params(self, d):
        languages = parse_http_accept_header(
            request.headers.get("Accept-Language")) or []
        language = d.get('language')
        if language and language != '*':
            languages.insert(0, language)
        languages = map(norm_tag, languages)
        default_language = norm_tag(self.default_language)
        languages.append(default_language)
        charsets = parse_http_accept_header(
            request.headers.get("Accept-Charset")) or []
        charsets = map(norm_charset, charsets)
        default_charset = norm_charset(self.default_charset)
        charset = norm_charset(config.get("kid.encoding"))
        if charset and charset != default_charset:
            charsets.append(charset)
        charsets.append(default_charset)
        path = "/tg_widgets/%s" % self.mod
        base_dir = config.get("static_filter.dir", path=path)
        base_name = "calendar/lang/calendar-"
        def find_link():
            params = dict()
            for lang in languages:
                if not lang or lang == '*':
                    lang = default_language
                base_lang = base_name + lang.replace('-', '_')
                for charset in charsets:
                    if not charset or charset == '*':
                        charset = default_charset
                    file_name = base_lang
                    if charset != default_charset:
                        file_name += '-' + charset.replace('-', '_')
                    file_name += '.js'
                    if os.path.exists(os.path.join(base_dir, file_name)):
                        path = "/%stg_widgets/%s" % (startup.webpath, self.mod)
                        link = "%s/%s" % (path, file_name)
                        return (link, language, charset)
                    if charset == default_charset:
                        break
        d["link"], d['language'], d['charset'] = find_link()



"""Definition of the identity predicates."""

import socket
import struct
import threading
import types
from turbogears import config
from cherrypy import request
from turbogears.identity.exceptions import *
from turbogears.identity import *
from turbogears.decorator import weak_signature_decorator


class Predicate(object):
    """Generic base class for testing true or false for a condition."""
    def eval_with_object(self, obj, errors=None):
        """Determine whether predicate is True or False for the given object."""
        raise NotImplementedError

    def append_error_message(self, errors=None):
        if errors is None:
            return
        errors.append(self.error_message % self.__dict__)


class CompoundPredicate(Predicate):
    """A predicate composed of other predicates."""
    def __init__(self, *predicates):
        self.predicates = predicates


class All(CompoundPredicate):
    """Logical and of all sub-predicates.

    This compound predicate evaluates to true only if all of its sub-predicates
    evaluate to true for the given input.

    """
    def eval_with_object(self, obj, errors=None):
        """Return true if all sub-predicates evaluate to true."""
        for p in self.predicates:
            if not p.eval_with_object(obj, errors):
                return False
        return True


class Any(CompoundPredicate):
    """Logical or of all sub-predicates.

    This compound predicate evaluates to true if any one of its sub-predicates
    evaluates to true for the given input.

    """
    error_message = "No predicates were able to grant access"

    def eval_with_object(self, obj, errors=None):
        """Return true if any sub-predicate evaluates to true.
        """
        for p in self.predicates:
            if p.eval_with_object(obj, None):
                return True
        self.append_error_message(errors)
        return False


class IdentityPredicateHelper(object):
    """A mix-in helper class for Identity Predicates."""
    def __nonzero__(self):
        return self.eval_with_object(current)


class in_group(Predicate, IdentityPredicateHelper):
    """Predicate for requiring a group."""
    error_message = "Not member of group: %(group_name)s"

    def __init__(self, group_name):
        self.group_name = group_name

    def eval_with_object(self, identity, errors=None):
        if self.group_name in identity.groups:
            return True
        self.append_error_message(errors)
        return False


class in_all_groups(All, IdentityPredicateHelper):
    """Predicate for requiring membership in a number of groups."""
    def __init__(self, *groups):
        group_predicates = [in_group(g) for g in groups]
        super(in_all_groups,self).__init__(*group_predicates)


class in_any_group(Any, IdentityPredicateHelper):
    """Predicate for requiring membership in at least one group."""
    error_message = "Not member of any group: %(group_list)s"

    def __init__(self, *groups):
        self.group_list = ", ".join(groups)
        group_predicates = [in_group(g) for g in groups]
        super(in_any_group,self).__init__(*group_predicates)


class not_anonymous(Predicate, IdentityPredicateHelper):
    """Predicate for checking whether current visitor is anonymous."""
    error_message = "Anonymous access denied"

    def eval_with_object(self, identity, errors=None):
        if identity.anonymous:
            self.append_error_message(errors)
            return False
        return True


class has_permission(Predicate, IdentityPredicateHelper):
    """Predicate for checking whether visitor has a particular permission."""
    error_message = "Permission denied: %(permission_name)s"

    def __init__(self, permission_name):
        self.permission_name = permission_name

    def eval_with_object(self, identity, errors=None):
        """Determine whether the visitor has the specified permission."""
        if self.permission_name in identity.permissions:
            return True
        self.append_error_message(errors)
        return False


class has_all_permissions(All, IdentityPredicateHelper):
    """Predicate for checking whether the visitor has all permissions."""
    def __init__(self, *permissions):
        permission_predicates = [has_permission(p) for p in permissions]
        super(has_all_permissions,self).__init__(*permission_predicates)


class has_any_permission(Any, IdentityPredicateHelper):
    """Predicate for checking whether visitor has at least one permission."""
    error_message = "No matching permissions: %(permission_list)s"

    def __init__(self, *permissions):
        self.permission_list = ', '.join(permissions)
        permission_predicates = [has_permission(p) for p in permissions]
        super(has_any_permission,self).__init__(*permission_predicates)


def _remoteHost():
    try:
        return request.headers.get('X-Forwarded-For', request.headers.get(
            'Remote-Addr', '')).split(',')[-1].strip() or None
    except AttributeError:
        raise RequestRequiredException()


def _match_ip(cidr, ip):
    if not '/' in cidr:
        return cidr == ip
    try:
        b, m = cidr.split('/')
        shift = 32 - int(m)
        a1 = struct.unpack('!L', socket.inet_aton(b))[0] >> shift
        a2 = struct.unpack('!L', socket.inet_aton(ip))[0] >> shift
        return a1 == a2
    except Exception:
        return False


class from_host(Predicate, IdentityPredicateHelper):
    """Predicate for checking whether the visitor's host is a permitted host.

    Note: We never want to announce what the list of allowed hosts is, because
    it is way too easy to spoof an IP address in a TCP/IP packet.

    """
    error_message = "Access from this host is not permitted."

    def __init__(self, host):
        self.host = host

    def eval_with_object(self, obj, errors=None):
        """Match the visitor's host against the criteria."""
        ip = _remoteHost()
        if ip and _match_ip(self.host, ip):
            return True
        self.append_error_message(errors)
        return False


class from_any_host(Any, IdentityPredicateHelper):
    """Predicate for checking the visitor against a number of allowed hosts."""
    error_message = "Access from this host is not permitted."

    def __init__(self, hosts):
        host_predicates = [from_host(h) for h in hosts]
        super(from_any_host, self).__init__(*host_predicates)


def require(predicate, obj=None):
    """Function decorator checking requirements for the current user.

    This function decorator checks whether the current user is a member
    of the groups specified and has the permissions required.

    """
    def entangle(fn):
        def require(func, self, *args, **kwargs):
            try:
                errors = []
                if predicate is None or \
                   predicate.eval_with_object(current, errors):
                    return fn(self, *args, **kwargs)
            except IdentityException, e:
                errors = [str(e)]
            raise IdentityFailure(errors)
        fn._require = predicate
        return require
    return weak_signature_decorator(entangle)


def _check_method(obj, fn, predicate):
    def _wrapper(*args, **kw):
        errors= []
        if predicate.eval_with_object(current, errors):
            return fn(*args, **kw)
        else:
            raise IdentityFailure(errors)
    _wrapper.exposed = True
    return _wrapper


class SecureResource(object):

    def __getattribute__(self, name):
        from turbogears import controllers
        if name[:3] == '_cp' or name == 'require':
            return object.__getattribute__(self,name)
        try:
            value = object.__getattribute__(self,name)
            try:
                predicate = object.__getattribute__(self, 'require')
            except AttributeError:
                predicate = config.get('identity.require', None)
            if predicate is None:
                raise AttributeError("SecureResource requires a 'require'"
                    " attribute either on the controller class itself"
                    " or in the config file")
            errors = []
            if (isinstance(value, types.MethodType) and
                    hasattr(value, 'exposed')):
                return _check_method(self, value, predicate)
            if isinstance(value, controllers.Controller):
                return SecureObject(value, predicate)
            # Some other property
            return value
        except IdentityException, e:
            errors= [str(e)]
        raise IdentityFailure(errors)


class SecureObject(object):

    def __init__(self, obj, require, exclude=[]):
        self._exclude = exclude
        self._object = obj
        self._require = require

    def __getattribute__(self, name):
        from turbogears import controllers
        if name[:3] == '_cp' or name in ('_object', '_require', '_exclude'):
            return object.__getattribute__(self,name)
        try:
            obj = object.__getattribute__(self, '_object')
            value = getattr(obj, name)
            errors = []
            predicate = object.__getattribute__(self, '_require')
            if name in object.__getattribute__(self, '_exclude'):
                return value
            if (isinstance(value, types.MethodType) and
                    hasattr(value, 'exposed')):
                return _check_method(obj, value, predicate)
            if isinstance(value, controllers.Controller):
                return SecureObject(value, predicate)
            # Some other property
            return value
        except IdentityException, e:
            errors = [str(e)]
        raise IdentityFailure(errors)



from turbogears.widgets.base import Widget, JSLink, static, mochikit
from turbojson import jsonify

__all__ = ["RPC"]

class RPC(Widget):
    """RPC base widget."""

    params = ["action", "update", "data", "on_success", "on_failure", "on_complete",
                     "before", "after", "loading", "loaded", "confirm"]

    javascript = [mochikit, JSLink(static,'ajax.js')]

    def update_params(self, d):
        super(RPC, self).update_params(d)

        d['js'] = "return ! remoteRequest(self, '%s', '%s', %s, %s)" % (
            d.get('action'),
            d.get('update'),
            jsonify.encode(d.get('data')),
            jsonify.encode(self.get_options(d)),
        )

    def get_options(self, d):
        return dict(
            on_success = d.get("on_success"),
            on_failure = d.get("on_failure"),
            on_complete = d.get("on_complete"),
            before = d.get("before"),
            after = d.get("after"),
            loading = d.get("loading"),
            loaded = d.get("loaded"),
            confirm = d.get("confirm"),
        )



from turbogears.visit.api import current
from turbogears.visit.api import enable_visit_plugin
from turbogears.visit.api import start_extension
from turbogears.visit.api import shutdown_extension
from turbogears.visit.api import create_extension_model



"""The TurboGears utility module."""

import os
import sys
import re
import logging
import warnings
import htmlentitydefs
from inspect import getargspec, getargvalues
from itertools import izip, islice, chain
from operator import isSequenceType
from urllib import quote, unquote

try:
    set
except NameError: # Python 2.3
    from sets import Set as set

import pkg_resources

from cherrypy import request

from turbogears.decorator import decorator
from turbogears import config


def deprecated(message=None):
    """Decorator which can be used to mark functions as deprecated.

    It will result in a warning being emmitted when the function is used.

    Inspired by http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/391367

    """
    def decorate(func):
        if not decorate.message:
            decorate.message = \
                "Call to deprecated function %s." % func.__name__
        def newFunc(*args, **kwargs):
            warnings.warn(decorate.message, category=DeprecationWarning,
                stacklevel = 2)
            return func(*args, **kwargs)
        try:
            newFunc.__name__ = func.__name__
        except TypeError: # not writable in Python 2.3
            pass
        newFunc.__doc__ = func.__doc__
        newFunc.__dict__.update(func.__dict__)
        return newFunc
    decorate.message = message
    return decorate


class Bunch(dict):
    """Simple but handy collector of a bunch of named stuff."""

    def __repr__(self):
        keys = self.keys()
        keys.sort()
        args = ', '.join(['%s=%r' % (key, self[key]) for key in keys])
        return '%s(%s)' % (self.__class__.__name__, args)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    __setattr__ = dict.__setitem__

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            raise AttributeError(name)


class DictObj(Bunch):

    def __init__(self, *args, **kw):
        super(DictObj, self).__init__(*args, **kw)
    __init__ = deprecated(
        "Use Bunch instead of DictObj and DictWrapper.")(__init__)

DictWrapper = DictObj


def Enum(*names):
    """True immutable symbolic enumeration with qualified value access.

    Written by Zoran Isailovski:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/413486

    """

    # Uncomment the following line if you don't like empty enums.
    # assert names, "Empty enums are not supported"

    class EnumClass(object):
        __slots__ = names
        def __iter__(self):
            return iter(constants)
        def __len__(self):
            return len(constants)
        def __getitem__(self, i):
            return constants[i]
        def __repr__(self):
            return 'Enum' + str(names)
        def __str__(self):
            return 'enum ' + str(constants)

    enumType = EnumClass()

    class EnumValue(object):
        __slots__ = ('__value')
        def __init__(self, value):
            self.__value = value
        Value = property(lambda self: self.__value)
        EnumType = property(lambda self: enumType)
        def __hash__(self):
            return hash(self.__value)
        def __cmp__(self, other):
            # C fans might want to remove the following assertion
            # to make all enums comparable by ordinal value {;))
            assert self.EnumType is other.EnumType, \
                "Only values from the same enum are comparable"
            return cmp(self.__value, other.__value)
        def __invert__(self):
            return constants[maximum - self.__value]
        def __nonzero__(self):
            return bool(self.__value)
        def __repr__(self):
            return str(names[self.__value])

    maximum = len(names) - 1
    constants = [None] * len(names)
    for i, each in enumerate(names):
        val = EnumValue(i)
        setattr(EnumClass, each, val)
        constants[i] = val
    constants = tuple(constants)
    return enumType


class setlike(list):
    """Set preserving item order."""

    def add(self, item):
        if item not in self:
            self.append(item)

    def add_all(self, iterable):
        for item in iterable:
            self.add(item)


def get_project_meta(name):
    """Get egg-info file with that name in the current project."""
    for dirname in os.listdir("./"):
        if dirname.lower().endswith("egg-info"):
            fname = os.path.join(dirname, name)
            return fname


def get_project_config():
    """Try to select appropriate project configuration file."""
    return os.path.exists('setup.py') and 'dev.cfg' or 'prod.cfg'


def load_project_config(configfile=None):
    """Try to update the project settings from the config file specified.

    If configfile is C{None}, uses L{get_project_config} to locate one.

    """
    if configfile is None:
        configfile = get_project_config()
    if not os.path.isfile(configfile):
        print 'Config file %s not found or is not a file.' % (
            os.path.abspath(configfile),)
        sys.exit()
    package = get_package_name()
    config.update_config(configfile=configfile,
        modulename = package + ".config")


def get_package_name():
    """Try to find out the package name of the current directory."""
    package = config.get("package")
    if package:
        return package
    if hasattr(sys, 'argv') and "--egg" in sys.argv:
        projectname = sys.argv[sys.argv.index("--egg")+1]
        egg = pkg_resources.get_distribution(projectname)
        top_level = egg._get_metadata("top_level.txt")
    else:
        fname = get_project_meta('top_level.txt')
        top_level = fname and open(fname) or []
    for package in top_level:
        package = package.rstrip()
        if package and package != 'locales':
            return package


def get_project_name():
    pkg_info = get_project_meta('PKG-INFO')
    if pkg_info:
        name = list(open(pkg_info))[1][6:-1]
        return name.strip()


def get_model():
    package_name = get_package_name()
    if not package_name:
        return None
    package = __import__(package_name, {}, {}, ["model"])
    if hasattr(package, "model"):
        return package.model


def ensure_sequence(obj):
    """Construct a sequence from object."""
    if obj is None:
        return []
    elif isSequenceType(obj):
        return obj
    else:
        return [obj]


def to_kw(func, args, kw, start=0):
    """Convert all applicable arguments to keyword arguments."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    kv_pairs = izip(
        islice(argnames, start, len(argnames) - len(defaults)), args)
    for k, v in kv_pairs:
        kw[k] = v
    return args[len(argnames)-len(defaults)-start:], kw


def from_kw(func, args, kw, start=0):
    """Extract named positional arguments from keyword arguments."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    newargs = [kw.pop(name) for name in islice(argnames, start,
        len(argnames) - len(defaults)) if name in kw]
    newargs.extend(args)
    return newargs, kw


def adapt_call(func, args, kw, start=0):
    """Remove excess arguments."""
    argnames, varargs, kwargs = getargspec(func)[:3]
    del argnames[:start]
    if kwargs in (None, "_decorator__kwargs"):
        remove_keys(kw, [key for key in kw if key not in argnames])
    if varargs in (None, "_decorator__varargs"):
        args = args[:len(argnames)]
    for n, key in enumerate(argnames):
        if key in kw:
            args = args[:n]
            break
    return args, kw


def call_on_stack(func_name, kw, start=0):
    """Check if a call to function matching pattern is on stack."""
    try:
        frame = sys._getframe(start+1)
    except ValueError:
        return False
    while frame.f_back:
        frame = frame.f_back
        if frame.f_code.co_name == func_name:
            args = getargvalues(frame)[3]
            for key in kw.iterkeys():
                try:
                    if kw[key] != args[key]:
                        break
                except (KeyError, TypeError):
                    break
            else:
                return True
    return False


def remove_keys(dict_, seq):
    """Gracefully remove keys from dict."""
    for key in seq:
        dict_.pop(key, None)
    return dict_


def has_arg(func, argname):
    """Check whether function has argument."""
    return argname in getargspec(func)[0]


def arg_index(func, argname):
    """Find index of argument as declared for given function."""
    argnames = getargspec(func)[0]
    if has_arg(func, argname):
        return argnames.index(argname)
    else:
        return None


def inject_arg(func, argname, argval, args, kw, start=0):
    """Insert argument into call."""
    argnames, defaults = getargspec(func)[::3]
    defaults = ensure_sequence(defaults)
    pos = arg_index(func, argname)
    if pos is None or pos > len(argnames) - len(defaults) - 1:
        kw[argname] = argval
    else:
        pos -= start
        args = tuple(chain(islice(args, pos), (argval,),
            islice(args, pos, None)))
    return args, kw


def inject_args(func, injections, args, kw, start=0):
    """Insert arguments into call."""
    for argname, argval in injections.iteritems():
        args, kw = inject_arg(func, argname, argval, args, kw, start)
    return args, kw


def inject_call(func, injections, *args, **kw):
    """Insert arguments and call."""
    args, kw = inject_args(func, injections, args, kw)
    return func(*args, **kw)


def add_tg_args(func, args):
    """Add hint for special arguments that shall not be removed."""
    try:
        tg_args = func._tg_args
    except AttributeError:
        tg_args = set()
    tg_args.update(args)
    func._tg_args = tg_args


def bind_args(**add):
    """Call with arguments set to a predefined value."""
    def entagle(func):
        return lambda func, *args, **kw: inject_call(func, add, *args, **kw)

    def make_decorator(func):
        argnames, varargs, kwargs, defaults = getargspec(func)
        defaults = list(ensure_sequence(defaults))
        defaults = [d for d in defaults if
            argnames[-len(defaults) + defaults.index(d)] not in add]
        argnames = [arg for arg in argnames if arg not in add]
        return decorator(entagle, (argnames, varargs, kwargs, defaults))(func)

    return make_decorator


def recursive_update(to_dict, from_dict):
    """Recursively update all dicts in to_dict with values from from_dict."""
    # probably slow as hell :( should be optimized somehow...
    for k, v in from_dict.iteritems():
        if isinstance(v, dict) and isinstance(to_dict[k], dict):
            recursive_update(to_dict[k], v)
        else:
            to_dict[k] = v
    return to_dict


def combine_contexts(frames=None, depth=None):
    """Combine contexts (globals, locals) of frames."""
    locals_ = {}
    globals_ = {}
    if frames is None:
        frames = []
    if depth is not None:
        frames.extend([sys._getframe(d+1) for d in depth])
    for frame in frames:
        locals_.update(frame.f_locals)
        globals_.update(frame.f_globals)
    return locals_, globals_


def request_available():
    """Check if cherrypy.request is available."""
    try:
        setattr(request, "tg_dumb_attribute", True)
        return True
    except AttributeError:
        return False


def flatten_sequence(seq):
    """Flatten sequence."""
    for item in seq:
        if isSequenceType(item) and not isinstance(item, basestring):
            for item in flatten_sequence(item):
                yield item
        else:
            yield item


def load_class(dottedpath):
    """Load a class from a module in dotted-path notation.

    E.g.: load_class("package.module.class").

    Based on recipe 16.3 from Python Cookbook, 2ed., by Alex Martelli,
    Anna Martelli Ravenscroft, and David Ascher (O'Reilly Media, 2005)

    """
    assert dottedpath is not None, "dottedpath must not be None"
    splitted_path = dottedpath.split('.')
    modulename = '.'.join(splitted_path[:-1])
    classname = splitted_path[-1]
    try:
        try:
            module = __import__(modulename, globals(), locals(), [classname])
        except ValueError: # Py < 2.5
            if not modulename:
                module = __import__(__name__.split('.')[0],
                    globals(), locals(), [classname])
    except ImportError:
        # properly log the exception information and return None
        # to tell caller we did not succeed
        logging.exception('tg.utils: Could not import %s'
            ' because exception occurred', dottedpath)
        return None
    return getattr(module, classname)


def parse_http_accept_header(accept):
    """Parse an HTTP Accept header (RFC 2616) into a sorted list.

    The quality factors in the header determine the sort order.
    The values can include possible media-range parameters.
    This function can also be used for the Accept-Charset,
    Accept-Encoding and Accept-Language headers.

    """
    if accept is None:
        return []
    items = []
    for item in accept.split(','):
        params = item.split(';')
        for i, param in enumerate(params[1:]):
            param = param.split('=', 1)
            if param[0].strip() == 'q':
                try:
                    q = float(param[1])
                    if not 0 < q <= 1:
                        raise ValueError
                except (IndexError, ValueError):
                    q = 0
                else:
                    item = ';'.join(params[:i+1])
                break
        else:
            q = 1
        if q:
            item = item.strip()
            if item:
                items.append((item, q))
    items.sort(lambda i1, i2: cmp(i2[1], i1[1]))
    return [item[0] for item in items]


def simplify_http_accept_header(accept):
    """Parse an HTTP Accept header (RFC 2616) into a preferred value.

    The quality factors in the header determine the preference.
    Possible media-range parameters are allowed, but will be ignored.
    This function can also be used for the Accept-Charset,
    Accept-Encoding and Accept-Language headers.

    This is similar to parse_http_accept_header(accept)[0], but faster.

    """
    if accept is None:
        return None
    best_item = accept
    best_q = 0
    for item in accept.split(','):
        params = item.split(';')
        item = params.pop(0)
        for param in params:
            param = param.split('=', 1)
            if param[0].strip() == 'q':
                try:
                    q = float(param[1])
                    if not 0 < q <= 1:
                        raise ValueError
                except (IndexError, ValueError):
                    q = 0
                break
        else:
            q = 1
        if q > best_q:
            item = item.strip()
            if item:
                best_item = item
                if q == 1:
                    break
                best_q = q
    return best_item


def to_unicode(value):
    """Convert encoded string to unicode string.

    Uses get_template_encoding_default() to guess source string encoding.
    Handles turbogears.i18n.lazystring correctly.

    """
    if isinstance(value, str):
        # try to make sure we won't get UnicodeDecodeError from the template
        # by converting all encoded strings to Unicode strings
        try:
            value = unicode(value)
        except UnicodeDecodeError:
            try:
                value = unicode(value, get_template_encoding_default())
            except UnicodeDecodeError:
                # fail early
                raise ValueError("Non-unicode string: %r" % value)
    return value


def to_utf8(value):
    """Convert a unicode string to utf-8 encoded plain string.

    Handles turbogears.i18n.lazystring correctly.

    Does nothing to already encoded string.

    """
    if isinstance(value, str):
        pass
    elif hasattr(value, '__unicode__'):
        value = unicode(value)
    if isinstance(value, unicode):
        value = value.encode('utf-8')
    return value


_safe_for_cookie = ''.join(filter(lambda c: c != quote(c)
    and c not in "$,; \t\f\n\r\v", map(chr, xrange(256))))

def quote_cookie(value):
    """Escape chars in value that should not be put directly in a cookie."""
    return quote(value, _safe_for_cookie)


def unquote_cookie(value):
    """Unescape chars in a cookie value, the reverse of quote_cookie(value)."""
    return unquote(value)


def get_template_encoding_default(engine_name=None):
    """Return default encoding for template files (Kid, Genshi, etc.)."""
    if engine_name is None:
        engine_name = config.get('tg.defaultview', 'kid')
    return config.get('%s.encoding' % engine_name,
        config.get('%s.default_encoding' % engine_name, 'utf-8'))


_format_mime_types = dict(
    plain='text/plain', text='text/plain',
    html='text/html', xhtml = 'text/html', # see note below
    xml='text/xml', json='application/json')

def get_mime_type_for_format(format):
    """Return default MIME media type for a template format.

    Note: By default we are serving xhtml as "text/html" instead of the more
    correct "application/xhtml+xml", since many browsers, particularly MSIE,
    do not support this. We are assuming that xhtml means XHTML 1.0 here,
    where this approach is possible. It would be possible to use some kind
    of content negotiation to deliver a customized content type, but we avoid
    this because it causes more harm (e.g. with proxies involved) than good.

    If you want to serve the proper content type (e.g. for XHTML 1.1),
    set tg.format_mime_types= {'xhtml': 'application/xhtml+xml'}.
    You can also set a particular content type per controller using the
    content_type parameter of the expose decorator.

    For detailed information about this issues, see here:
    http://www.smackthemouse.com/xhtmlxml, http://schneegans.de/web/xhtml/.

    """
    mime_type = config.get('tg.format_mime_types', {}).get(format)
    if not mime_type:
        mime_type = _format_mime_types.get(format, 'text/html')
    return mime_type


def mime_type_has_charset(mime_type):
    """Return whether the MIME media type supports a charset parameter.

    Note: According to RFC4627, we do not output a charset parameter
    for "application/json" (this type always uses a UTF encoding).

    """
    if not mime_type:
        return False
    if mime_type.startswith('text/'):
        return True
    if mime_type.startswith('application/'):
        if mime_type.endswith('/xml') or mime_type.endswith('+xml'):
            return True
        if mime_type.endswith('/javascript'):
            return True
    return False


def find_precision(value):
    """Find precision of some arbitrary value.

    The main intention for this function is to use it together with
    turbogears.i18n.format.format_decimal() where one has to inform
    the precision wanted. So, use it like this:

    format_decimal(some_number, find_precision(some_number))

    """
    decimals = ''
    try:
        decimals = str(value).split('.', 1)[1]
    except IndexError:
        pass
    return len(decimals)


def copy_if_mutable(value, feedback=False):
    """Make a copy of the value if it is mutable.

    Returns the value. If feedback is set to true, also returns
    whether value was mutable as the second element of a tuple.

    """
    if isinstance(value, dict):
        mutable = True
        value = value.copy()
    elif isinstance(value, list):
        mutable = True
        value = value[:]
    else:
        mutable = False
    if feedback:
        return value, mutable
    else:
        return value


def fixentities(htmltext):
    """Replace HTML character entities with numerical references.

    Note: This won't handle CDATA sections properly.

    """
    def repl(matchobj):
        entity = htmlentitydefs.entitydefs.get(matchobj.group(1).lower())
        if not entity:
            return matchobj.group(0)
        elif len(entity) == 1:
            if entity in "&<>'\"":
                return matchobj.group(0)
            return "&#%d;" % ord(entity)
        else:
            return entity
    return re.sub("&(\w+);?", repl, htmltext)


__all__ = ["Bunch", "DictObj", "DictWrapper", "Enum", "setlike",
           "get_package_name", "get_model", "load_project_config",
           "ensure_sequence", "has_arg", "to_kw", "from_kw", "adapt_call",
           "call_on_stack", "remove_keys", "arg_index",
           "inject_arg", "inject_args", "add_tg_args", "bind_args",
           "recursive_update", "combine_contexts",
           "request_available", "flatten_sequence", "load_class",
           "parse_http_accept_header", "simplify_http_accept_header",
           "to_unicode", "to_utf8", "quote_cookie", "unquote_cookie",
           "get_template_encoding_default", "get_mime_type_for_format",
           "mime_type_has_charset", "find_precision", "copy_if_mutable",
           "deprecated"]



"""Classes and methods for TurboGears controllers."""

import logging
import re
import urllib
import urlparse
import types
from itertools import izip
from dispatch import generic, strategy, functions
from inspect import isclass
import cherrypy
from cherrypy import request, response
import turbogears.util as tg_util
from turbogears import view, database, errorhandling, config
from turbogears.decorator import weak_signature_decorator
from turbogears.validators import Invalid
from turbogears.errorhandling import error_handler, exception_handler


log = logging.getLogger("turbogears.controllers")


if config.get("session_filter.on", None):
    if config.get("session_filter.storage_type", None) == "PostgreSQL":
        import psycopg2
        config.update(
                {'session_filter.get_db': psycopg2.connect(
                    psycopg2.get('sessions.postgres.dsn'))
                    })
    # support for mysql/sqlite/etc here


def _process_output(output, template, format, content_type,
        mapping, fragment=False):
    """Produce final output form from data returned from a controller method.

    See the expose() arguments for more info since they are the same.

    """
    if isinstance(output, dict):
        from turbogears.widgets import js_location

        css = tg_util.setlike()
        js = dict(izip(js_location, iter(tg_util.setlike, None)))
        include_widgets = {}
        include_widgets_lst = config.get("tg.include_widgets", [])

        if config.get("tg.mochikit_all", False):
            include_widgets_lst.insert(0, 'turbogears.mochikit')

        for i in include_widgets_lst:
            widget = tg_util.load_class(i)
            if isclass(widget):
                widget = widget()
            include_widgets["tg_%s" % i.split(".")[-1]] = widget
            for script in widget.retrieve_javascript():
                if hasattr(script, "location"):
                    js[script.location].add(script)
                else:
                    js[js_location.head].add(script)
            css.add_all(widget.retrieve_css())

        for value in output.itervalues():
            if hasattr(value, "retrieve_css"):
                retrieve = getattr(value, "retrieve_css")
                if callable(retrieve):
                    css.add_all(value.retrieve_css())
            if hasattr(value, "retrieve_javascript"):
                retrieve = getattr(value, "retrieve_javascript")
                if callable(retrieve):
                    for script in value.retrieve_javascript():
                        if hasattr(script, "location"):
                            js[script.location].add(script)
                        else:
                            js[js_location.head].add(script)
        output.update(include_widgets)
        output["tg_css"] = css
        for location in iter(js_location):
            output["tg_js_%s" % str(location)] = js[location]

        tg_flash = _get_flash()
        if tg_flash:
            output["tg_flash"] = tg_flash

        headers = {'Content-Type': content_type}
        output = view.render(output, template=template, format=format,
                    mapping=mapping, headers=headers,
                    fragment=fragment)
        content_type = headers['Content-Type']

    if content_type:
        response.headers["Content-Type"] = content_type
    else:
        content_type = response.headers.get("Content-Type", 'text/plain')

    if content_type.startswith("text/"):
        if isinstance(output, unicode):
            output = output.encode(tg_util.get_template_encoding_default())

    return output


class BadFormatError(Exception):
    """Output-format exception."""


def validate(form=None, validators=None,
             failsafe_schema=errorhandling.FailsafeSchema.none,
             failsafe_values=None, state_factory=None):
    """Validate input.

    @param form: a form instance that must be passed throught the validation
    process... you must give a the same form instance as the one that will
    be used to post data on the controller you are putting the validate
    decorator on.
    @type form: a form instance

    @param validators: individual validators to use for parameters.
    If you use a schema for validation then the schema instance must
    be the sole argument.
    If you use simple validators, then you must pass a dictionary with
    each value name to validate as a key of the dictionary and the validator
    instance (eg: tg.validators.Int() for integer) as the value.
    @type validators: dictionary or schema instance

    @param failsafe_schema: a schema for handling failsafe values.
    The default is 'none', but you can also use 'values', 'map_errors',
    or 'defaults' to map erroneous inputs to values, corresponding exceptions
    or method defaults.
    @type failsafe_schema: errorhandling.FailsafeSchema

    @param failsafe_values: replacements for erroneous inputs. You can either
    define replacements for every parameter, or a single replacement value
    for all parameters. This is only used when failsafe_schema is 'values'.
    @type failsafe_values: a dictionary or a single value

    @param state_factory: If this is None, the initial state for validation
    is set to None, otherwise this must be a callable that returns the initial
    state to be used for validation.
    @type state_factory: callable or None

    """
    def entangle(func):
        if callable(form) and not hasattr(form, "validate"):
            init_form = lambda self: form(self)
        else:
            init_form = lambda self: form

        def validate(func, *args, **kw):
            # do not validate a second time if already validated
            if hasattr(request, 'validation_state'):
                return func(*args, **kw)

            form = init_form(args and args[0] or kw["self"])
            args, kw = tg_util.to_kw(func, args, kw)

            errors = {}
            if state_factory is not None:
                state = state_factory()
            else:
                state = None

            if form:
                value = kw.copy()
                try:
                    kw.update(form.validate(value, state))
                except Invalid, e:
                    errors = e.unpack_errors()
                    request.validation_exception = e
                request.validated_form = form

            if validators:
                if isinstance(validators, dict):
                    for field, validator in validators.iteritems():
                        try:
                            kw[field] = validator.to_python(
                                kw.get(field, None), state)
                        except Invalid, error:
                            errors[field] = error
                else:
                    try:
                        value = kw.copy()
                        kw.update(validators.to_python(value, state))
                    except Invalid, e:
                        errors = e.unpack_errors()
                        request.validation_exception = e
            request.validation_errors = errors
            request.input_values = kw.copy()
            request.validation_state = state

            if errors:
                kw = errorhandling.dispatch_failsafe(failsafe_schema,
                                            failsafe_values, errors, func, kw)
            args, kw = tg_util.from_kw(func, args, kw)
            return errorhandling.run_with_errors(errors, func, *args, **kw)

        return validate
    return weak_signature_decorator(entangle)


class CustomDispatch(functions.GenericFunction):

    def combine(self, cases):
        strict = [strategy.ordered_signatures, strategy.safe_methods]
        cases = strategy.separate_qualifiers(
            cases,
            primary = strict,
        )
        primary = strategy.method_chain(cases.get('primary', []))
        if type(primary) != types.FunctionType:
            for i in primary:
                for y in i:
                    return y[1]
        return primary


def _add_rule(_expose, found_default, as_format, accept_format, template,
              rulefunc):
    if as_format == "default":
        if found_default:
            colon = template.find(":")
            if colon == -1:
                as_format = template
            else:
                as_format = template[:colon]
        else:
            found_default = True
    ruleparts = ['kw.get("tg_format", "default") == "%s"' % as_format]
    if accept_format:
        ruleparts.append('(accept == "%s" and kw.get("tg_format"'
            ', "default") == "default")' % accept_format)
    rule = " or ".join(ruleparts)
    log.debug("Generated rule %s", rule)
    _expose.when(rule)(rulefunc)

    return found_default


def _build_rules(func):
    [generic(CustomDispatch)]
    def _expose(func, accept, allow_json, *args, **kw):
        pass

    if func._allow_json:
        log.debug("Adding allow_json rule: "
            'allow_json and (kw.get("tg_format", None) == "json"'
            ' or accept in ("application/json", "text/javascript"))')
        _expose.when('allow_json and (kw.get("tg_format", None) == "json"'
            ' or accept in ("application/json", "text/javascript"))')(
            lambda _func, accept, allow_json, *args, **kw:
                _execute_func(_func, "json", "json", "application/json",
                    None, False, args, kw))

    found_default = False
    for ruleinfo in func._ruleinfo:
        found_default = _add_rule(_expose, found_default, **ruleinfo)

    func._expose = _expose


def expose(template=None, validators=None, allow_json=None, html=None,
           format=None, content_type=None, inputform=None, fragment=False,
           as_format="default", mapping=None, accept_format=None):
    """Exposes a method to the web.

    By putting the expose decorator on a method, you tell TurboGears that
    the method should be accessible via URL traversal. Additionally, expose
    handles the output processing (turning a dictionary into finished
    output) and is also responsible for ensuring that the request is
    wrapped in a database transaction.

    You can apply multiple expose decorators to a method, if
    you'd like to support multiple output formats. The decorator that's
    listed first in your code without as_format or accept_format is
    the default that is chosen when no format is specifically asked for.
    Any other expose calls that are missing as_format and accept_format
    will have as_format implicitly set to the whatever comes before
    the ":" in the template name (or the whole template name if there
    is no ":". For example, <code>expose("json")</code>, if it's not
    the default expose, will have as_format set to "json".

    When as_format is set, passing the same value in the tg_format
    parameter in a request will choose the options for that expose
    decorator. Similarly, accept_format will watch for matching
    Accept headers. You can also use both. expose("json", as_format="json",
    accept_format="application/json") will choose JSON output for either
    case: tg_format=json as a parameter or Accept: application/json as a
    request header.

    Passing allow_json=True to an expose decorator
    is equivalent to adding the decorator just mentioned.

    Each expose decorator has its own set of options, and each one
    can choose a different template or even template engine (you can
    use Kid for HTML output and Cheetah for plain text, for example).
    See the other expose parameters below to learn about the options
    you can pass to the template engine.

    Take a look at the
    <a href="tests/test_expose-source.html">test_expose.py</a> suite
    for more examples.

    @param template "templateengine:dotted.reference" reference along the
            Python path for the template and the template engine. For
            example, "kid:foo.bar" will have Kid render the bar template in
            the foo package.
    @keyparam format format for the template engine to output (if the
            template engine can render different formats. Kid, for example,
            can render "html", "xml" or "xhtml")
    @keyparam content_type sets the content-type http header
    @keyparam allow_json allow the function to be exposed as json
    @keyparam fragment for template engines (like Kid) that generate
            DOCTYPE declarations and the like, this is a signal to
            just generate the immediate template fragment. Use this
            if you're building up a page from multiple templates or
            going to put something onto a page with .innerHTML.
    @keyparam mapping mapping with options that are sent to the template
            engine
    @keyparam as_format designates which value of tg_format will choose
            this expose.
    @keyparam accept_format which value of an Accept: header will
            choose this expose.
    @keyparam html deprecated in favor of template
    @keyparam validators deprecated. Maps argument names to validator
            applied to that arg
    @keyparam inputform deprecated. A form object that generates the
            input to this method

    """
    if html:
        template = html
    if not template:
        template = format
    if format == "json" or (format is None and template is None):
        template = "json"
        allow_json = True
    if content_type is None:
        content_type = config.get("tg.content_type", None)

    if config.get("tg.session.automatic_lock", None):
        cherrypy.session.acquire_lock()

    def entangle(func):
        log.debug("Exposing %s", func)
        log.debug("template: %s, format: %s, allow_json: %s, "
            "content-type: %s", template, format, allow_json, content_type)
        if not getattr(func, "exposed", False):
            def expose(func, *args, **kw):
                accept = request.headers.get('Accept', "").lower()
                accept = tg_util.simplify_http_accept_header(accept)
                if not hasattr(func, "_expose"):
                    _build_rules(func)
                if hasattr(request, "in_transaction"):
                    output = func._expose(func, accept, func._allow_json,
                                *args, **kw)
                else:
                    request.in_transaction = True
                    output = database.run_with_transaction(
                            func._expose, func, accept, func._allow_json,
                            *args, **kw)
                return output
            func.exposed = True
            func._ruleinfo = []
            allow_json_from_config = config.get("tg.allow_json", False)
            func._allow_json = allow_json_from_config or template == 'json'
        else:
            expose = lambda func, *args, **kw: func(*args, **kw)

        func._ruleinfo.insert(0, dict(as_format=as_format,
            accept_format=accept_format, template=template,
            rulefunc = lambda _func, accept, allow_json, *args, **kw:
                _execute_func(_func, template, format, content_type,
                    mapping, fragment, args, kw)))

        if allow_json:
            func._allow_json = True

        if inputform or validators:
            import warnings
            warnings.warn(
                "Use a separate decorator validate() rather than passing "
                "arguments validators and/or inputform to decorator "
                "expose().",
                DeprecationWarning, 2)
            func = validate(form=inputform, validators=validators)(func)

        return expose
    return weak_signature_decorator(entangle)


def _execute_func(func, template, format, content_type, mapping, fragment, args, kw):
    """Call controller method and process it's output."""
    if config.get("tg.strict_parameters", False):
        tg_util.remove_keys(kw, ["tg_random", "tg_format"]
            + config.get("tg.ignore_parameters", []))
    else:
        # get special parameters used by upstream decorators like paginate
        try:
            tg_kw = dict([(k, v) for k, v in kw.items() if k in func._tg_args])
        except AttributeError:
            tg_kw = {}
        # remove excessive parameters
        args, kw = tg_util.adapt_call(func, args, kw)
        # add special parameters again
        kw.update(tg_kw)
    if config.get('server.environment', 'development') == 'development':
        # Only output this in development mode: If it's a field storage object,
        # this means big memory usage, and we don't want that in production
        log.debug("Calling %s with *(%s), **(%s)", func, args, kw)
    output = errorhandling.try_call(func, *args, **kw)
    assert isinstance(output, (basestring, dict, list, types.GeneratorType)), \
           "Method %s.%s() returned unexpected output. Output should " \
           "be of type basestring, dict, list or generator." % (
            args[0].__class__.__name__, func.__name__)
    if isinstance(output, dict):
        template = output.pop("tg_template", template)
        format = output.pop("tg_format", format)
    if template and template.startswith("."):
        template = func.__module__[:func.__module__.rfind('.')]+template
    return _process_output(output, template, format, content_type, mapping, fragment)


def flash(message):
    """Set a message to be displayed in the browser on next page display."""
    message = tg_util.quote_cookie(tg_util.to_utf8(message))
    response.simple_cookie['tg_flash'] = message
    response.simple_cookie['tg_flash']['path'] = '/'


def _get_flash():
    """Retrieve the flash message (if one is set), clearing the message."""
    request_cookie = request.simple_cookie
    response_cookie = response.simple_cookie

    def clearcookie():
        response_cookie["tg_flash"] = ""
        response_cookie["tg_flash"]['expires'] = 0
        response_cookie['tg_flash']['path'] = '/'

    if response_cookie.has_key("tg_flash"):
        message = response_cookie["tg_flash"].value
        response_cookie.pop("tg_flash")
        if request_cookie.has_key("tg_flash"):
            # New flash overrided old one sitting in cookie. Clear that old cookie.
            clearcookie()
    elif request_cookie.has_key("tg_flash"):
        message = request_cookie["tg_flash"].value
        if not response_cookie.has_key("tg_flash"):
            clearcookie()
    else:
        message = None
    if message:
        message = unicode(tg_util.unquote_cookie(message), 'utf-8')
    return message


class Controller(object):
    """Base class for a web application's controller.

    Currently, this provides positional parameters functionality
    via a standard default method.

    """


class RootController(Controller):
    """Base class for the root of a web application.

    Your web application should have one of these. The root of
    your application is used to compute URLs used by your app.

    """
    is_app_root = True

    msglog = logging.getLogger('cherrypy.msg')
    msglogfunc = {0: msglog.info, 1: msglog.warning, 2: msglog.error}
    def _cp_log_message(self, msg, context = 'nocontext', severity = 0):
        log = self.msglogfunc[severity]
        text = ''.join((context, ': ', msg))
        log(text)

    accesslog = logging.getLogger("turbogears.access")
    def _cp_log_access(self):
        tmpl = '%(h)s %(l)s %(u)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
        try:
            username = request.user_name
            if username:
                if isinstance(username, unicode):
                    # might get concatenated to non-ascii byte strings
                    username = username.encode('utf-8')
            else:
                username = "-"
        except AttributeError:
            username = "-"
        s = tmpl % {'h': request.remote_host,
                   'l': '-',
                   'u': username,
                   'r': request.requestLine,
                   's': response.status.split(" ", 1)[0],
                   'b': response.headers.get('Content-Length',
                            '') or "-",
                   'f': request.headers.get('referer', ''),
                   'a': request.headers.get('user-agent', ''),
        }
        self.accesslog.info(s)

Root = RootController


def url(tgpath, tgparams=None, **kw):
    """Computes URLs.

    tgpath can be a list or a string. If the path is absolute (starts
    with a "/"), the server.webpath, SCRIPT_NAME and the approot of the
    application are prepended to the path. In order for the approot to
    be detected properly, the root object should extend
    controllers.RootController.

    Query parameters for the URL can be passed in as a dictionary in
    the second argument *or* as keyword parameters.

    Values which are a list or a tuple are used to create multiple
    key-value pairs.

    """
    if not isinstance(tgpath, basestring):
        tgpath = '/'.join(list(tgpath))
    if tgpath.startswith('/'):
        webpath = (config.get('server.webpath') or '').rstrip('/')
        if tg_util.request_available():
            check_app_root()
            tgpath = request.app_root + tgpath
            try:
                webpath += request.wsgi_environ['SCRIPT_NAME'].rstrip('/')
            except (AttributeError, KeyError):
                pass
        tgpath = webpath + tgpath
    if tgparams is None:
        tgparams = kw
    else:
        try:
            tgparams = tgparams.copy()
            tgparams.update(kw)
        except AttributeError:
            raise TypeError('url() expects a dictionary for query parameters')
    args = []
    for key, value in tgparams.iteritems():
        if value is None:
            continue
        if isinstance(value, (list, tuple)):
            pairs = [(key, v) for v in value]
        else:
            pairs = [(key, value)]
        for k, v in pairs:
            if v is None:
                continue
            if isinstance(v, unicode):
                v = v.encode('utf8')
            args.append((k, str(v)))
    if args:
        query_string = urllib.urlencode(args, True)
        if '?' in tgpath:
            tgpath += '&' + query_string
        else:
            tgpath += '?' + query_string
    return tgpath


def check_app_root():
    """Sets request.app_root if needed."""
    if hasattr(request, 'app_root'):
        return
    found_root = False
    trail = request.object_trail
    top = len(trail) - 1
    # compute the app_root by stepping back through the object
    # trail and collecting up the path elements after the first
    # root we find
    # we can eliminate this if we find a way to use
    # CherryPy's mounting mechanism whenever a new root
    # is hit.
    rootlist = []
    for i in xrange(len(trail) - 1, -1, -1):
        path, obj = trail[i]
        if not found_root and isinstance(obj, RootController):
            if i == top:
                break
            found_root = True
        if found_root and i > 0:
            rootlist.insert(0, path)
    app_root = '/'.join(rootlist)
    if not app_root.startswith('/'):
        app_root = '/' + app_root
    if app_root.endswith('/'):
        app_root = app_root[:-1]
    request.app_root = app_root


def redirect(redirect_path, redirect_params=None, **kw):
    """Redirect (via cherrypy.HTTPRedirect).

    Raises the exception instead of returning it, this to allow
    users to both call it as a function or to raise it as an exception.

    """
    if not isinstance(redirect_path, basestring):
        redirect_path = '/'.join(list(redirect_path))
    if not redirect_path.startswith('/'):
        path = request.path_info
        check_app_root()
        if path.startswith(request.app_root):
            path = path[len(request.app_root):]
        redirect_path = urlparse.urljoin(path, redirect_path)
    raise cherrypy.HTTPRedirect(url(tgpath=redirect_path,
        tgparams=redirect_params, **kw))


__all__ = [
    "Controller",
    "error_handler",
    "exception_handler",
    "expose",
    "flash",
    "redirect",
    "Root",
    "RootController",
    "url",
    "validate",
]



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

import sys
import os

if sys.version_info < (2, 3):
    raise SystemExit("TurboGears 1.0 requires Python 2.3 or later.")
elif sys.version_info >= (2, 6):
    # The main problem here is RuleDispatch.
    # TurboGears 1.1 uses PEAK-Rules instead.
    import warnings
    warnings.warn("TurboGears 1.0 does not support Python 2.6 or later."
        "\nYou should upgrade to TurboGears 1.1 or later.")

# import meta data (version, author etc.)
execfile(os.path.join("turbogears", "release.py"))

# setup params
install_requires = [
    "CherryPy >= 2.3.0, < 3.0dev",
    "ConfigObj >= 4.3.2",
    "DecoratorTools >= 1.4",
    "FormEncode >= 0.7.1",
    "PasteScript >= 1.6.2",
    "RuleDispatch >= 0.5a0.dev-r2303",
    "setuptools >= 0.6c2",
    "simplejson >= 1.3",
    "TurboCheetah >= 1.0",
    # TurboJson >= 1.2 uses PEAK-Rules
    "TurboJson >= 1.1.4, < 1.2",
    "TurboKid >= 1.0.5",
]

exp = ["TGFastData"]

future = [
    "Elixir >= 0.4.0",
    "Genshi >= 0.4.4",
]

testtools =  [
    "nose >= 0.9.3",
]

tgtesttools = testtools

if sys.version_info < (2, 5):
    # simplejson >= 2.1 does not support Py 2.4
    install_requires[install_requires.index(
        "simplejson >= 1.3")] += ", < 2.1dev"
    # Python < 2.5 does not include SQLite
    tgtesttools.append("pysqlite < 2.6dev")

if sys.version_info < (2, 4):
    # FormEcode >= 1.3 does not support Py 2.3
    install_requires[install_requires.index(
        "FormEncode >= 0.7.1")] += ", < 1.3dev"
    # Genshi >= 0.5 does not support Py 2.3
    future[future.index(
        "Genshi >= 0.4.4")] += ", < 0.5dev"
    sqlobject = [
        # SQLObject >= 0.11 does not support Py 2.3
        "SQLObject >= 0.10.1, < 0.11dev",
        ]
    sqlalchemy = [
        # SQLAlchemy >= 0.5 does not support Py 2.3
        "SQLAlchemy >= 0.3.10, < 0.5dev",
        ]
else:
    sqlobject = [
        "SQLObject >= 0.10.1, < 0.12dev",
    ]
    sqlalchemy = [
        "SQLAlchemy >= 0.3.10, < 0.6dev",
    ]

develop_requires = (install_requires
    + future + tgtesttools + sqlalchemy + sqlobject)

setup(
    name="TurboGears",
    description=description,
    long_description=long_description,
    version=version,
    author=author,
    author_email=email,
    maintainer=maintainer,
    maintainer_email=maintainer_email,
    url=url,
    download_url=download_url,
    dependency_links=dependency_links,
    license=license,
    zip_safe=False,
    install_requires=install_requires,
    packages=find_packages(),
    include_package_data=True,
    exclude_package_data={"thirdparty": ["*"]},
    entry_points="""
    [console_scripts]
    tg-admin = turbogears.command:main

    [distutils.commands]
    docs = turbogears.docgen:GenSite

    [paste.paster_create_template]
    tgbase = turbogears.command.quickstart:BaseTemplate
    turbogears = turbogears.command.quickstart:TurbogearsTemplate
    tgbig = turbogears.command.quickstart:TGBig
    tgwidget = turbogears.command.quickstart:TGWidgetTemplate

    [turbogears.command]
    quickstart = turbogears.command.quickstart:quickstart
    sql = turbogears.command.base:SQL
    shell = turbogears.command.base:Shell
    toolbox = turbogears.command.base:ToolboxCommand
    update = turbogears.command.quickstart:update
    i18n = turbogears.command.i18n:InternationalizationTool
    info = turbogears.command.info:InfoCommand

    [turbogears.identity.provider]
    sqlobject = turbogears.identity.soprovider:SqlObjectIdentityProvider
    sqlalchemy= turbogears.identity.saprovider:SqlAlchemyIdentityProvider

    [turbogears.extensions]
    identity = turbogears.identity.visitor
    visit = turbogears.visit

    [turbogears.visit.manager]
    sqlobject = turbogears.visit.sovisit:SqlObjectVisitManager
    sqlalchemy = turbogears.visit.savisit:SqlAlchemyVisitManager

    [turbogears.toolboxcommand]
    widgets = turbogears.toolbox.base:WidgetBrowser
    shell = turbogears.toolbox.shell:WebConsole
    admi18n = turbogears.toolbox.admi18n:Internationalization
    designer = turbogears.toolbox.designer:Designer
    info = turbogears.toolbox.base:Info
    catwalk = turbogears.toolbox.catwalk:CatWalk

    """,
    extras_require={
        "exp" : exp,
        "sqlobject" : sqlobject,
        "sqlalchemy" : sqlalchemy,
        "future" : future,
        "testtools" : testtools,
        "tgtesttools" : tgtesttools,
        "develop" : develop_requires
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    test_suite='nose.collector',
)



"""Tests for SQLObject database operations"""

from turbogears import database
from sqlobject import dbconnection
import cherrypy
from turbogears.controllers import RootController


_engine = database._engine

def setup(module):
    # turn off SQLAlchemy for these tests
    global _engine
    _engine = database._engine
    database._engine = None

def teardown(module):
    database._engine = _engine


def test_registry():
    """hubs appear in a registry"""
    hub = database.AutoConnectHub("sqlite:///:memory:")
    try:
        assert hub in database.hub_registry
    finally:
        database.hub_registry.clear()


def test_alwaysTransaction():
    """hub.getConnection always returns a Transaction"""
    hub = database.AutoConnectHub("sqlite:///:memory:")
    try:
        assert isinstance(hub.getConnection(), dbconnection.Transaction)
        database.end_all()
        assert not isinstance(hub.threadingLocal.connection, dbconnection.Transaction)
    finally:
        database.hub_registry.clear()


class DatabaseStandIn:

    committed = False
    rolled_back = False
    ended = False
    successful_called = False

    def commit(self):
        self.committed = True

    def rollback(self):
        self.rolled_back = True

    def end(self):
        self.ended = True

    def successful(self):
        self.successful_called = True

    def failure(self):
        self.failure_called = True
        raise Warning("Oh my!")

    def redirect(self):
        self.redirect_called = True
        raise cherrypy.HTTPRedirect("/")


def test_good_transaction():
    """successful runs automatically commit"""
    dsi = DatabaseStandIn()
    database.hub_registry.add(dsi)
    try:
        database.run_with_transaction(dsi.successful)
        assert dsi.successful_called
        assert dsi.committed
        assert dsi.ended
    finally:
        database.hub_registry.clear()


def test_bad_transaction():
    """failed runs automatically rollback"""
    dsi = DatabaseStandIn()
    database.hub_registry.add(dsi)
    try:
        try:
            database.run_with_transaction(dsi.failure)
            assert False, "exception should have been raised"
        except:
            pass
        assert dsi.failure_called
        assert dsi.rolled_back
        assert dsi.ended
    finally:
        database.hub_registry.clear()


def test_redirection():
    """Redirects count as successful runs, not failures"""
    dsi = DatabaseStandIn()
    database.hub_registry.add(dsi)
    try:
        try:
            database.run_with_transaction(dsi.redirect)
        except cherrypy.HTTPRedirect:
            pass
        assert dsi.redirect_called
        assert dsi.committed
        assert dsi.ended
    finally:
        database.hub_registry.clear()


def test_so_to_dict():
    """Conversion to dictionary works properly"""
    hub = database.AutoConnectHub("sqlite:///:memory:")
    try:
        from sqlobject import IntCol
        from sqlobject.inheritance import InheritableSQLObject

        class Parent(InheritableSQLObject):
            _connection = hub
            a   = IntCol()

        class Child(Parent):
            _connection = hub
            b   = IntCol()

        Parent.createTable()
        Child.createTable()
        p =  Parent(a=1)
        c =  Child(a=1, b=2)

        p_dict = database.so_to_dict(p)
        assert p_dict['a'] == 1

        c_dict = database.so_to_dict(c)
        assert c_dict['a'] == 1
        assert c_dict['b'] == 2
        assert None == c_dict.get('childName', None)
        assert None == p_dict.get('childName', None)
    finally:
        database.hub_registry.clear()



#!/usr/bin/env python
names = ["aces", "astonishing", "astounding",
    "best", "breathtaking", "cool",
    "doozie", "extravagant", "fab", "fantastic",
    "groovy",
    "immense", "in spades", "inconceivable",
    "legendary", "marvelous", "mind-blowing", "out-of-this-world",
    "outrageous", "phenomenal", "primo", "prodigious",
    "remarkable", "spectacular", "striking",
    "super", "superb", "terrific", "top drawer", "tops", "turn-on",
    "unbelievable", "wicked", "wondrous"]

import random
print random.choice(names)



"""\
Front-to-back rapid web development
===================================

TurboGears brings together four major pieces to create an easy to
install, easy to use web mega-framework. It covers everything from
front end (MochiKit JavaScript for the browser, Genshi / Kid / Mako /
Cheetah for templates in Python) to the controllers (CherryPy) to the
back end (SQLAlchemy or SQLObject).

The TurboGears project is focused on providing documentation and
integration with these tools without losing touch with the communities
that already exist around those tools.

TurboGears is easy to use for a wide range of web applications.

The latest development version is available in the `TurboGears
subversion repository`_.

Our `mailing list`_ is lively and helpful, don't hesitate to send your
questions there, we will try to help you find out a solution to your
problem.

.. _mailing list:
    http://groups.google.com/group/turbogears

.. _TurboGears subversion repository:
    http://svn.turbogears.org/trunk#egg=turbogears-dev
"""

version = "1.0.10"
description = "Front-to-back, open-source, rapid web development framework"
long_description = __doc__
author = "Kevin Dangoor"
email = "dangoor+turbogears@gmail.com"
maintainer = "TurboGears Release Team"
maintainer_email = "turbogears@googlegroups.com"
url = "http://www.turbogears.org/"
download_url = "http://www.turbogears.org/%s/downloads/%s/index" % (
    '.'.join(version.split('.', 2)[:2]), version)
dependency_links = [download_url]
copyright = "Copyright 2005 - 2011 Kevin Dangoor and contributors"
license = "MIT"



# Formatting configuration for locale fo

languages={'fo': u'f\xf8royskt'}

countries={'FO': u'F\xf8royar'}

months=['januar', 'februar', 'mars', u'apr\xedl', 'mai', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'mai', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'des']

days=[u'm\xe1nadagur', u't\xfdsdagur', 'mikudagur', u'h\xf3sdagur', u'fr\xedggjadagur', 'leygardagur', 'sunnudagur']

abbrDays=[u'm\xe1n', u't\xfds', 'mik', u'h\xf3s', u'fr\xed', 'ley', 'sun']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d. %%(abbrmonthname)s %Y', 'short': '%d-%m-%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}



"""
General i18n utility functions.
"""

import urllib

import simplejson
import cherrypy

from turbogears import config
from turbogears.release import version as tg_version
from turbogears.util import parse_http_accept_header, request_available

class TGURLopener(urllib.FancyURLopener):
    version = "TurboGears/%s" % tg_version

def google_translate(from_lang, to_lang, text):
    """Translate text via the translate.google.com service.

    The source language is given by 'from_lang' and the target language as
    'to_lang'. 'text' must be a unicode or UTF-8 encoded string.

    """
    if isinstance(text, unicode):
        has_nbsp = u'\xa0' in text
        text = text.encode('utf-8')
    else:
        has_nbsp = False
    params = urllib.urlencode(dict(v='1.0',
        langpair='%s|%s' % (from_lang, to_lang), q=text))
    try:
        result = TGURLopener().open('http://ajax.googleapis.com'
            '/ajax/services/language/translate', params).read()
    except IOError:
        text = None
    else:
        try:
            result = simplejson.loads(result)
        except ValueError:
            text = None
        else:
            try:
                text = result['responseData']['translatedText']
            except (KeyError, TypeError):
                text = None
            else:
                if text and not has_nbsp:
                    text = text.replace(u'\xa0', ' ')
    return text

def lang_in_gettext_format(lang):
    if len(lang) > 2:
        country = lang[3:].upper()
        lang = lang[:2] + "_" + country
    return lang

def get_accept_languages(accept):
    """Returns a list of languages, by order of preference, based on an
    HTTP Accept-Language string.See W3C RFC 2616
    (http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html) for specification.
    """
    langs = parse_http_accept_header(accept)
    for index, lang in enumerate(langs):
        langs[index] = lang_in_gettext_format(lang)
    return langs

def get_locale(locale=None):
    """
    Returns user locale, using _get_locale or app-specific locale lookup function.
    """
    if not locale:
        get_locale_f = config.get("i18n.get_locale", _get_locale)
        locale = get_locale_f()
    return locale

def _get_locale():
    """Default function for returning locale. First looks in session for locale key,
    then checks the HTTP Accept-Language header, and finally checks the config default
    locale setting. This can be replaced by your own function by setting cherrypy
    config setting i18n.get_locale to your function name.
    """
    if not request_available():
        return config.get("i18n.default_locale", "en")

    if config.get("session_filter.on", False):
        locale_key = config.get("i18n.session_key", "locale")
        locale = cherrypy.session.get(locale_key)
        if locale:
            return locale
    browser_accept_lang = _get_locale_from_accept_header()
    return browser_accept_lang or config.get("i18n.default_locale", "en")

def _get_locale_from_accept_header():
    """
    Checks HTTP Accept-Language header to find preferred language if any.
    """
    try:
        header = cherrypy.request.headers.get("Accept-Language")
        if header:
            accept_languages = get_accept_languages(header)
            if accept_languages:
                return accept_languages[0]
    except AttributeError:
        pass

def set_session_locale(locale):
    """
    Sets the i18n session locale.

    Raises an error if session support is not enabled.
    """
    cherrypy.session[config.get("i18n.session_key", "locale")] = locale



import logging
import ntpath
import os
import re
import sys

from cStringIO import StringIO

import pkg_resources
import turbogears

rfn = pkg_resources.resource_filename
testfile = rfn(__name__, "configfile.cfg")

logout = StringIO()
logging.logout = logout

# last forward slash (the one before static) is hard coded in our config
# file... all other path separators are calculated platform wise...

def test_update_from_package():
    turbogears.update_config(modulename="turbogears.tests.config")
    assert turbogears.config.get("foo.bar") == "BAZ!"
    print turbogears.config.get("my.static")
    assert turbogears.config.get("my.static").endswith(
            "turbogears%stests/static" % os.path.sep)
    assert turbogears.config.get("static_filter.on", path="/static") == True

def test_update_from_both():
    turbogears.update_config(configfile = testfile,
        modulename="turbogears.tests.config")
    print turbogears.config.get("foo.bar")
    assert turbogears.config.get("foo.bar") == "blurb"
    assert turbogears.config.get("tg.something") == 10
    print turbogears.config.get("test.dir")
    assert turbogears.config.get("test.dir").endswith(
        "turbogears%stests" % os.path.sep)

callnum = 0

def windows_filename(*args, **kw):
    """Small helper function to emulate pkg_resources.resource_filename
    as if it was called on a Wwindows system even if the tester is in fact
    using Linux or Mac OS X.

    We need to keep track how often the function was called, since
    'turbogears.update_config' calls 'pkg_resources.resource_filename' at least
    twice and we onyl want to return the fake Windows path the second and
    following times.

    """
    global callnum
    callnum += 1
    if callnum > 1:
        return "c:\\foo\\bar\\"
    else:
        return rfn(*args, **kw)

def test_update_on_windows():
    """turbogears.update_config works as we intend on Windows.
    """
    # save the original function
    orig_resource_fn = pkg_resources.resource_filename
    # monkey patch pkg resources to emulate windows
    pkg_resources.resource_filename = windows_filename

    turbogears.update_config(configfile=testfile,
        modulename="turbogears.tests.config")
    testdir = turbogears.config.get("test.dir")
    # update_config calls os.normpath on package_dir, but this will have no
    # effect on non-windows systems, so we call ntpath.normpath on those here
    if not sys.platform.startswith('win'):
        testdir = ntpath.normpath(testdir)

    # restore original function
    pkg_resources.resource_filename = orig_resource_fn
    assert testdir == "c:\\foo\\bar"

def test_logging_config():
    logout.truncate(0)
    log = logging.getLogger("turbogears.tests.test_config.logconfig")
    log.info("Testing")
    logged = logout.getvalue()
    print "Logged: %s" % logged
    assert re.match(r'F1 \d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d,\d\d\d INFO '
                    'Testing', logged)
    assert turbogears.config.get("tg.new_style_logging", False)



# Formatting configuration for locale et_EE

languages={'el': 'Kreeka', 'en': 'Inglise', 'zh': 'Hiina', 'it': 'Itaalia', 'ar': 'Araabia', 'cs': 'Tiehhi', 'et': 'Eesti', 'es': 'Hispaania', 'ru': 'Vene', 'nl': 'Hollandi', 'pt': 'Portugali', 'no': 'Norra', 'tr': u'T\xfcrgi', 'lv': u'L\xe4ti', 'lt': 'Leedu', 'ro': 'Rumeenia', 'pl': 'Poola', 'fr': 'Prantsuse', 'bg': 'Bulgaaria', 'hr': 'Horvaadi', 'de': 'Saksa', 'da': 'Taani', 'fi': 'Soome', 'hu': 'Ungari', 'ja': 'Jaapani', 'he': 'Heebrea', 'ko': 'Korea', 'sv': 'Rootsi', 'sk': 'Slovaki', 'sl': 'Sloveeni'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaaria', 'BA': 'Bosnia ja Hertsegoviina', 'BB': 'Barbados', 'WF': 'Wallis ja Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Boliivia', 'BH': 'Bahrein', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': "Bouvet' saar", 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasiilia', 'BS': 'Bahama saared', 'BY': 'Valgevene', 'BZ': 'Belize', 'RU': 'Venemaa', 'RW': 'Rwanda', 'TL': 'Ida-Timor', 'RE': u'R\xe9union', 'TM': u'T\xfcrkmenistan', 'TJ': u'Tad\u012aikistan', 'RO': 'Rumeenia', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'L\xf5una-Georgia ja L\xf5una-Sandwichi saared', 'GR': 'Kreeka', 'GQ': 'Ekvatoriaal-Guinea', 'GP': 'Guadeloupe', 'JP': 'Jaapan', 'GY': 'Guyana', 'GF': 'Prantsuse Guajaana', 'GE': 'Gruusia', 'GD': 'Grenada', 'GB': u'\xdchendkuningriik', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6\xf6nimaa', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Omaan', 'TN': 'Tuneesia', 'JO': 'Jordaania', 'SP': 'Serbia', 'HR': 'Horvaatia', 'HT': 'Haiti', 'HU': 'Ungari', 'HK': 'Hongkongi erihalduspiirkond', 'HN': 'Honduras', 'HM': 'Heard ja McDonald', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': 'Palestiina Omavalitsus', 'PW': 'Belau', 'PT': 'Portugal', 'SJ': 'Svalbard ja Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraak', 'PA': 'Panama', 'PF': u'Prantsuse Pol\xfcneesia', 'PG': 'Paapua Uus-Guinea', 'PE': 'Peruu', 'PK': 'Pakistan', 'PH': 'Filipiinid', 'PN': 'Pitcairn', 'PL': 'Poola', 'PM': 'Saint-Pierre ja Miquelon', 'ZM': 'Sambia', 'EH': u'L\xe4\xe4ne-Sahara', 'EE': 'Eesti', 'EG': 'Egiptus', 'ZA': u'L\xf5una-Aafrika Vabariik', 'EC': 'Ecuador', 'IT': 'Itaalia', 'VN': 'Vietnam', 'SB': 'Saalomoni Saared', 'ET': 'Etioopia', 'SO': 'Somaalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi Araabia', 'ES': 'Hispaania', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Maroko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Aomeni Hiina erihalduspiirkond', 'MN': 'Mongoolia', 'MH': 'Marshalli Saared', 'MK': 'Makedoonia Vabariik', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldiivid', 'MQ': 'Martinique', 'MP': u'P\xf5hja-Mariaanid', 'MS': 'Montserrat', 'MR': 'Mauritaania', 'UG': 'Uganda', 'MY': 'Malaisia', 'MX': 'Mehhiko', 'IL': 'Iisrael', 'FR': 'Prantsusmaa', 'IO': 'Briti India ookeani ala', 'SH': 'Saint Helena', 'FI': 'Soome', 'FJ': u'Fid\u012ai', 'FK': 'Falklandi saared', 'FM': 'Mikroneesia Liiduriigid', 'FO': u'F\xe4\xe4ri saared', 'NI': 'Nicaragua', 'NL': 'Holland', 'NO': 'Norra', 'NA': 'Namiibia', 'VU': 'Vanuatu', 'NC': 'Uus-Kaledoonia', 'NE': 'Niger', 'NF': 'Norfolk', 'NG': 'Nigeeria', 'NZ': 'Uus-Meremaa', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cooki saared', 'CI': "Cote d'Ivoire", 'CH': u'\u0128veits', 'CO': 'Colombia', 'CN': 'Hiina', 'CM': 'Kamerun', 'CL': 'Tiiili', 'CC': 'Kookossaared', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Kesk-Aafrika Vabariik', 'CD': 'Kongo DV', 'CZ': 'Tiehhi Vabariik', 'CY': u'K\xfcpros', 'CX': u'J\xf5ulusaar', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Cabo Verde', 'CU': 'Kuuba', 'SZ': 'Svaasimaa', 'SY': u'S\xfc\xfcria', 'KG': u'K\xf5rg\xf5zstan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': u'Kambod\u012aa', 'KN': 'Saint Kitts ja Nevis', 'KM': 'Komoorid', 'ST': u'Sao Tom\xe9 ja Principe', 'SK': 'Slovakkia', 'KR': u'L\xf5una-Korea', 'SI': 'Sloveenia', 'KP': u'P\xf5hja-Korea', 'KW': 'Kuveit', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seiiellid', 'KZ': 'Kasahstan', 'KY': 'Kaimani saared', 'SG': 'Singapur', 'SE': 'Rootsi', 'SD': 'Sudaan', 'DO': 'Dominikaani Vabariik', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Taani', 'VG': 'Briti Neitsisaared', 'DE': 'Saksamaa', 'YE': 'Jeemen', 'DZ': u'Al\u012aeeria', 'US': u'Ameerika \xdchendriigid', 'UY': 'Uruguay', 'YU': 'Jugoslaavia', 'YT': 'Mayotte', 'UM': u'\xdchendriikide hajasaared', 'LB': 'Liibanon', 'LC': 'Saint Lucia', 'LA': 'Laose DRV', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad ja Tobago', 'TR': u'T\xfcrgi', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': u'L\xe4ti', 'TO': 'Tonga', 'LT': 'Leedu', 'LU': 'Luksemburg', 'LR': 'Libeeria', 'LS': 'Lesotho', 'TH': 'Tai', 'TF': u'Prantsuse L\xf5unaalad', 'TG': 'Togo', 'TD': 'Tiaad', 'TC': 'Turks ja Caicos', 'LY': u'Liib\xfca', 'VA': u'P\xfcha Tool (Vatikan)', 'VC': 'Saint Vincent ja Grenadiinid', 'AE': u'Araabia \xdchendemiraadid', 'AD': 'Andorra', 'AG': 'Antigua ja Barbuda', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'USA Neitsisaared', 'IS': 'Island', 'IR': 'Iraan', 'AM': 'Armeenia', 'AL': 'Albaania', 'AO': 'Angola', 'AN': 'Hollandi Antillid', 'AQ': 'Antarktika', 'AS': 'Ameerika Samoa', 'AR': 'Argentina', 'AU': 'Austraalia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tansaania', 'AZ': u'Aserbaid\u012aaan', 'IE': 'Iirimaa', 'ID': 'Indoneesia', 'UA': 'Ukraina', 'QA': 'Katar', 'MZ': 'Mosambiik'}

months=['jaanuar', 'veebruar', u'm\xe4rts', 'aprill', 'mai', 'juuni', 'juuli', 'august', 'september', 'oktoober', 'november', 'detsember']

abbrMonths=['jaan', 'veebr', u'm\xe4rts', 'apr', 'mai', 'juuni', 'juuli', 'aug', 'sept', 'okt', 'nov', 'dets']

days=[u'esmasp\xe4ev', u'teisip\xe4ev', u'kolmap\xe4ev', u'neljap\xe4ev', 'reede', u'laup\xe4ev', u'p\xfchap\xe4ev']

abbrDays=['E', 'T', 'K', 'N', 'R', 'L', 'P']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d, %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
